/*     */ package com.netapp.asup.data.function.aggregation;
/*     */ 
/*     */ import com.netapp.asup.data.function.FunctionAttribute;
/*     */ import com.netapp.asup.data.function.InputType;
/*     */ import com.netapp.asup.data.function.RegisteredFunction;
/*     */ import com.netapp.asup.data.function.TypeConverter;
/*     */ import com.netapp.asup.data.function.handler.StringOutputHandler;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.generic.GenericArray;
/*     */ import org.apache.avro.generic.GenericData.Array;
/*     */ import org.apache.avro.generic.GenericData.Record;
/*     */ import org.apache.avro.generic.GenericRecord;
/*     */ import org.apache.avro.util.Utf8;
/*     */ import org.codehaus.commons.compiler.CompileException;
/*     */ import org.codehaus.janino.ExpressionEvaluator;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class AggregationInputImpl
/*     */   implements AggregationInput
/*     */ {
/*     */   private final Map<String, Map<String, AggregationField>> object;
/*     */   
/*     */   public static enum Options
/*     */   {
/*  31 */     DELIMITER("delimiter");
/*     */     
/*     */     private final String name;
/*     */     private static final Map<String, Options> optionMap;
/*     */     
/*     */     static
/*     */     {
/*  38 */       optionMap = new HashMap();
/*  39 */       for (Options type : values()) {
/*  40 */         optionMap.put(type.getName(), type);
/*     */       }
/*     */     }
/*     */     
/*     */     private Options(String name) {
/*  45 */       this.name = name;
/*     */     }
/*     */     
/*     */     public String getName() {
/*  49 */       return this.name;
/*     */     }
/*     */     
/*     */     public static Options getOption(String name) {
/*  53 */       return (Options)optionMap.get(name);
/*     */     }
/*     */   }
/*     */   
/*  57 */   public AggregationInputImpl() { this.object = new HashMap(); }
/*     */   
/*     */ 
/*     */   static class AggregationField
/*     */   {
/*     */     private final AggregationType aggregationFunction;
/*     */     
/*     */     private final RegisteredFunction<?, ?> customFunction;
/*     */     
/*     */     private final String[] attributes;
/*     */     
/*     */     private final Class<?>[] attributeTypes;
/*     */     
/*     */     private final String expresssion;
/*     */     
/*     */     private final Map<String, String> options;
/*     */     
/*     */     private final InputType aggregatorType;
/*     */     
/*     */     private AggregationField(AggregationType aggregationFunction, String attribute, Map<String, String> options)
/*     */     {
/*  78 */       this.customFunction = null;
/*  79 */       this.aggregationFunction = aggregationFunction;
/*  80 */       this.attributes = new String[1];
/*  81 */       this.attributes[0] = attribute;
/*  82 */       this.attributeTypes = null;
/*  83 */       this.expresssion = null;
/*  84 */       this.options = options;
/*  85 */       this.aggregatorType = InputType.FIELD;
/*     */     }
/*     */     
/*     */     private AggregationField(AggregationType aggregationFunction, RegisteredFunction<?, ?> customFunction, Map<String, String> options) {
/*  89 */       this.customFunction = customFunction;
/*  90 */       this.aggregationFunction = aggregationFunction;
/*  91 */       this.expresssion = null;
/*  92 */       this.attributes = null;
/*  93 */       this.attributeTypes = null;
/*  94 */       this.options = options;
/*  95 */       this.aggregatorType = InputType.FUNCTION;
/*     */     }
/*     */     
/*     */     private AggregationField(AggregationType aggregationFunction, String expression, Map<String, String> options, FunctionAttribute... attributeTypePairs) throws IllegalArgumentException {
/*  99 */       this.customFunction = null;
/* 100 */       this.aggregationFunction = aggregationFunction;
/* 101 */       this.expresssion = expression;
/* 102 */       if (attributeTypePairs != null)
/*     */       {
/* 104 */         this.attributes = new String[attributeTypePairs.length];
/* 105 */         this.attributeTypes = new Class[attributeTypePairs.length];
/*     */         
/* 107 */         for (int index = 0; index < attributeTypePairs.length; index++) {
/* 108 */           this.attributes[index] = attributeTypePairs[index].getAttrName();
/*     */           
/* 110 */           this.attributeTypes[index] = attributeTypePairs[index].getAttrType();
/*     */         }
/*     */       }
/*     */       else {
/* 114 */         throw new IllegalArgumentException("Invalid expression " + expression + ", no attributes defined");
/*     */       }
/*     */       
/* 117 */       this.options = options;
/* 118 */       this.aggregatorType = InputType.EXPRESSION;
/*     */       try {
/* 120 */         new ExpressionEvaluator(expression, Object.class, this.attributes, this.attributeTypes);
/*     */       } catch (CompileException e) {
/* 122 */         throw new IllegalArgumentException("Invalid expression " + expression + ", caused by :" + e.getCause(), e);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public GenericRecord getAggregationFieldObject(Schema aggregationFieldSchema)
/*     */     {
/* 132 */       GenericRecord aggregationField = new GenericData.Record(aggregationFieldSchema);
/* 133 */       aggregationField.put("aggregationFunction", new Utf8(this.aggregationFunction.getName()));
/* 134 */       loadAttributes(aggregationFieldSchema, aggregationField);
/* 135 */       loadAttributeTypes(aggregationFieldSchema, aggregationField);
/* 136 */       if (this.expresssion != null) {
/* 137 */         aggregationField.put("expression", new Utf8(this.expresssion));
/*     */       }
/* 139 */       if (this.customFunction != null) {
/* 140 */         aggregationField.put("customFunction", new Utf8(this.customFunction.getFunctionName()));
/*     */       }
/* 142 */       loadOptions(aggregationField);
/* 143 */       aggregationField.put("aggregatorType", this.aggregatorType.getName());
/* 144 */       return aggregationField;
/*     */     }
/*     */     
/*     */     private void loadOptions(GenericRecord aggregationField) {
/* 148 */       Map<Utf8, Utf8> options = new HashMap();
/* 149 */       if (this.options != null) {
/* 150 */         for (Map.Entry<String, String> entry : this.options.entrySet())
/*     */         {
/* 152 */           String optionName = (String)entry.getKey();
/*     */           
/* 154 */           options.put(new Utf8(optionName), new Utf8((String)entry.getValue()));
/*     */         }
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 160 */       aggregationField.put("options", options);
/*     */     }
/*     */     
/*     */     private void loadAttributeTypes(Schema aggregationFieldSchema, GenericRecord aggregationField)
/*     */     {
/* 165 */       GenericArray<Utf8> attributeTypes = null;
/* 166 */       if (this.attributeTypes != null) {
/* 167 */         attributeTypes = new GenericData.Array(this.attributeTypes.length, aggregationFieldSchema.getField("attributeTypes").schema());
/* 168 */         for (Class<?> attributeType : this.attributeTypes) {
/* 169 */           if (TypeConverter.getConvertorclassmap().containsKey(attributeType)) {
/* 170 */             attributeTypes.add(new Utf8(attributeType.getName()));
/*     */           } else {
/* 172 */             throw new IllegalArgumentException("Invalid attributeType. Type :" + attributeType.getName() + ", not supported");
/*     */           }
/*     */         }
/*     */       }
/*     */       else {
/* 177 */         attributeTypes = new GenericData.Array(0, aggregationFieldSchema.getField("attributeTypes").schema());
/*     */       }
/* 179 */       aggregationField.put("attributeTypes", attributeTypes);
/*     */     }
/*     */     
/*     */     private void loadAttributes(Schema aggregationFieldSchema, GenericRecord aggregationField)
/*     */     {
/* 184 */       GenericArray<Utf8> attributes = null;
/* 185 */       if (this.attributes != null) {
/* 186 */         attributes = new GenericData.Array(this.attributes.length, aggregationFieldSchema.getField("attributes").schema());
/* 187 */         for (String attribute : this.attributes) {
/* 188 */           attributes.add(new Utf8(attribute));
/*     */         }
/*     */       } else {
/* 191 */         attributes = new GenericData.Array(0, aggregationFieldSchema.getField("attributes").schema());
/*     */       }
/* 193 */       aggregationField.put("attributes", attributes);
/*     */     }
/*     */   }
/*     */   
/*     */   public Map<String, AggregationField> getDerivedAttributeMap(String path) {
/* 198 */     path = path.replaceFirst("^/", "");
/* 199 */     Map<String, AggregationField> map = (Map)this.object.get(path);
/* 200 */     if (map == null) {
/* 201 */       map = new HashMap();
/* 202 */       this.object.put(path, map);
/*     */     }
/* 204 */     return map;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void addAttribute(String path, String derivedAttribute, AggregationType aggregationFunction, String attribute, Map<String, String> options)
/*     */   {
/* 212 */     Map<String, AggregationField> derivedAttributeMap = getDerivedAttributeMap(path);
/* 213 */     derivedAttributeMap.put(derivedAttribute, new AggregationField(aggregationFunction, attribute, options, null));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public <I, O> void addCustomFunction(String path, String derivedAttribute, AggregationType aggregationFunction, RegisteredFunction<I, String> customFunction, Map<String, String> options)
/*     */   {
/* 221 */     Map<String, AggregationField> derivedAttributeMap = getDerivedAttributeMap(path);
/* 222 */     if ((customFunction.getOutputHandler() != null) && ((customFunction.getOutputHandler() instanceof StringOutputHandler))) {
/* 223 */       derivedAttributeMap.put(derivedAttribute, new AggregationField(aggregationFunction, customFunction, options, null));
/*     */     } else {
/* 225 */       throw new IllegalArgumentException("Unsupported custom function :" + customFunction.getFunctionName() + " , custom function should have a StringOutputHandler");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void addExpression(String path, String derivedAttribute, AggregationType aggregationFunction, String expression, Map<String, String> options, FunctionAttribute... attributeType)
/*     */   {
/* 235 */     Map<String, AggregationField> derivedAttributeMap = getDerivedAttributeMap(path);
/* 236 */     derivedAttributeMap.put(derivedAttribute, new AggregationField(aggregationFunction, expression, options, attributeType, null));
/*     */   }
/*     */   
/*     */ 
/*     */   public static Map<String, String> initializeOptionPair(String name, String value)
/*     */   {
/* 242 */     Map<String, String> options = new HashMap();
/* 243 */     options.put(name, value);
/* 244 */     return options;
/*     */   }
/*     */   
/*     */   public Map<String, Map<String, AggregationField>> getObject() {
/* 248 */     return this.object;
/*     */   }
/*     */   
/*     */ 
/*     */   public void setObject(Map<Utf8, Map<Utf8, AggregationField>> object)
/*     */   {
/* 254 */     for (Utf8 key : object.keySet())
/*     */     {
/* 256 */       Map<Utf8, AggregationField> aggrFieldMap = (Map)object.get(key);
/*     */       
/* 258 */       Map<String, AggregationField> aggrMap = new HashMap();
/* 259 */       for (Utf8 aggrKey : aggrFieldMap.keySet()) {
/* 260 */         aggrMap.put(aggrKey.toString(), aggrFieldMap.get(aggrKey));
/*     */       }
/* 262 */       this.object.put(key.toString(), aggrMap);
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\function\aggregation\AggregationInputImpl.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */