/*     */ package com.sun.xml.bind.v2.runtime;
/*     */ 
/*     */ import com.sun.istack.FinalArrayList;
/*     */ import com.sun.xml.bind.Util;
/*     */ import com.sun.xml.bind.api.AccessorException;
/*     */ import com.sun.xml.bind.v2.ClassFactory;
/*     */ import com.sun.xml.bind.v2.model.core.ID;
/*     */ import com.sun.xml.bind.v2.model.runtime.RuntimeClassInfo;
/*     */ import com.sun.xml.bind.v2.model.runtime.RuntimePropertyInfo;
/*     */ import com.sun.xml.bind.v2.runtime.property.AttributeProperty;
/*     */ import com.sun.xml.bind.v2.runtime.property.Property;
/*     */ import com.sun.xml.bind.v2.runtime.property.PropertyFactory;
/*     */ import com.sun.xml.bind.v2.runtime.reflect.Accessor;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.Loader;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.StructureLoader;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.UnmarshallingContext;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.XsiTypeLoader;
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.InvocationTargetException;
/*     */ import java.lang.reflect.Method;
/*     */ import java.lang.reflect.Modifier;
/*     */ import java.util.Collection;
/*     */ import java.util.Collections;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.logging.Level;
/*     */ import java.util.logging.Logger;
/*     */ import javax.xml.bind.helpers.ValidationEventImpl;
/*     */ import javax.xml.namespace.QName;
/*     */ import javax.xml.stream.XMLStreamException;
/*     */ import org.xml.sax.Locator;
/*     */ import org.xml.sax.SAXException;
/*     */ import org.xml.sax.helpers.LocatorImpl;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class ClassBeanInfoImpl<BeanT>
/*     */   extends JaxBeanInfo<BeanT>
/*     */ {
/*     */   public final Property<BeanT>[] properties;
/*     */   private Property<? super BeanT> idProperty;
/*     */   private Loader loader;
/*     */   private Loader loaderWithTypeSubst;
/*     */   private RuntimeClassInfo ci;
/*     */   private final Accessor<? super BeanT, Map<QName, String>> inheritedAttWildcard;
/*     */   private final Transducer<BeanT> xducer;
/*     */   public final ClassBeanInfoImpl<? super BeanT> superClazz;
/*     */   private final Accessor<? super BeanT, Locator> xmlLocatorField;
/*     */   private final Name tagName;
/*     */   private AttributeProperty<BeanT>[] attributeProperties;
/*     */   private Property<BeanT>[] uriProperties;
/*     */   private final Method factoryMethod;
/*     */   
/*     */   ClassBeanInfoImpl(JAXBContextImpl owner, RuntimeClassInfo ci)
/*     */   {
/*  96 */     super(owner, ci, (Class)ci.getClazz(), ci.getTypeName(), ci.isElement(), false, true);
/*     */     
/*  98 */     this.ci = ci;
/*  99 */     this.inheritedAttWildcard = ci.getAttributeWildcard();
/* 100 */     this.xducer = ci.getTransducer();
/* 101 */     this.factoryMethod = ci.getFactoryMethod();
/*     */     
/* 103 */     if (this.factoryMethod != null) {
/* 104 */       int classMod = this.factoryMethod.getDeclaringClass().getModifiers();
/*     */       
/* 106 */       if ((!Modifier.isPublic(classMod)) || (!Modifier.isPublic(this.factoryMethod.getModifiers()))) {
/*     */         try
/*     */         {
/* 109 */           this.factoryMethod.setAccessible(true);
/*     */         }
/*     */         catch (SecurityException e) {
/* 112 */           logger.log(Level.FINE, "Unable to make the method of " + this.factoryMethod + " accessible", e);
/* 113 */           throw e;
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 119 */     if (ci.getBaseClass() == null) {
/* 120 */       this.superClazz = null;
/*     */     } else {
/* 122 */       this.superClazz = owner.getOrCreate(ci.getBaseClass());
/*     */     }
/* 124 */     if ((this.superClazz != null) && (this.superClazz.xmlLocatorField != null)) {
/* 125 */       this.xmlLocatorField = this.superClazz.xmlLocatorField;
/*     */     } else {
/* 127 */       this.xmlLocatorField = ci.getLocatorField();
/*     */     }
/*     */     
/* 130 */     Collection<? extends RuntimePropertyInfo> ps = ci.getProperties();
/* 131 */     this.properties = new Property[ps.size()];
/* 132 */     int idx = 0;
/* 133 */     boolean elementOnly = true;
/* 134 */     for (RuntimePropertyInfo info : ps) {
/* 135 */       Property p = PropertyFactory.create(owner, info);
/* 136 */       if (info.id() == ID.ID)
/* 137 */         this.idProperty = p;
/* 138 */       this.properties[(idx++)] = p;
/* 139 */       elementOnly &= info.elementOnlyContent();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 144 */     hasElementOnlyContentModel(elementOnly);
/*     */     
/*     */ 
/* 147 */     if (ci.isElement()) {
/* 148 */       this.tagName = owner.nameBuilder.createElementName(ci.getElementName());
/*     */     } else {
/* 150 */       this.tagName = null;
/*     */     }
/* 152 */     setLifecycleFlags();
/*     */   }
/*     */   
/*     */   protected void link(JAXBContextImpl grammar)
/*     */   {
/* 157 */     if (this.uriProperties != null) {
/* 158 */       return;
/*     */     }
/* 160 */     super.link(grammar);
/*     */     
/* 162 */     if (this.superClazz != null) {
/* 163 */       this.superClazz.link(grammar);
/*     */     }
/* 165 */     getLoader(grammar, true);
/*     */     
/*     */ 
/* 168 */     if (this.superClazz != null) {
/* 169 */       if (this.idProperty == null) {
/* 170 */         this.idProperty = this.superClazz.idProperty;
/*     */       }
/* 172 */       if (!this.superClazz.hasElementOnlyContentModel()) {
/* 173 */         hasElementOnlyContentModel(false);
/*     */       }
/*     */     }
/*     */     
/* 177 */     List<AttributeProperty> attProps = new FinalArrayList();
/* 178 */     List<Property> uriProps = new FinalArrayList();
/* 179 */     for (ClassBeanInfoImpl bi = this; bi != null; bi = bi.superClazz) {
/* 180 */       for (int i = bi.properties.length - 1; i >= 0; i--) {
/* 181 */         Property p = bi.properties[i];
/* 182 */         if ((p instanceof AttributeProperty))
/* 183 */           attProps.add((AttributeProperty)p);
/* 184 */         if (p.hasSerializeURIAction())
/* 185 */           uriProps.add(p);
/*     */       }
/*     */     }
/* 188 */     if (grammar.c14nSupport) {
/* 189 */       Collections.sort(attProps);
/*     */     }
/* 191 */     if (attProps.isEmpty()) {
/* 192 */       this.attributeProperties = EMPTY_PROPERTIES;
/*     */     } else {
/* 194 */       this.attributeProperties = ((AttributeProperty[])attProps.toArray(new AttributeProperty[attProps.size()]));
/*     */     }
/* 196 */     if (uriProps.isEmpty()) {
/* 197 */       this.uriProperties = EMPTY_PROPERTIES;
/*     */     } else
/* 199 */       this.uriProperties = ((Property[])uriProps.toArray(new Property[uriProps.size()]));
/*     */   }
/*     */   
/*     */   public void wrapUp() {
/* 203 */     for (Property p : this.properties)
/* 204 */       p.wrapUp();
/* 205 */     this.ci = null;
/* 206 */     super.wrapUp();
/*     */   }
/*     */   
/*     */   public String getElementNamespaceURI(BeanT bean) {
/* 210 */     return this.tagName.nsUri;
/*     */   }
/*     */   
/*     */   public String getElementLocalName(BeanT bean) {
/* 214 */     return this.tagName.localName;
/*     */   }
/*     */   
/*     */   public BeanT createInstance(UnmarshallingContext context) throws IllegalAccessException, InvocationTargetException, InstantiationException, SAXException
/*     */   {
/* 219 */     BeanT bean = null;
/* 220 */     if (this.factoryMethod == null) {
/* 221 */       bean = ClassFactory.create0(this.jaxbType);
/*     */     } else {
/* 223 */       Object o = ClassFactory.create(this.factoryMethod);
/* 224 */       if (this.jaxbType.isInstance(o)) {
/* 225 */         bean = (BeanT)o;
/*     */       } else {
/* 227 */         throw new InstantiationException("The factory method didn't return a correct object");
/*     */       }
/*     */     }
/*     */     
/* 231 */     if (this.xmlLocatorField != null)
/*     */       try
/*     */       {
/* 234 */         this.xmlLocatorField.set(bean, new LocatorImpl(context.getLocator()));
/*     */       } catch (AccessorException e) {
/* 236 */         context.handleError(e);
/*     */       }
/* 238 */     return bean;
/*     */   }
/*     */   
/*     */   public boolean reset(BeanT bean, UnmarshallingContext context) throws SAXException {
/*     */     try {
/* 243 */       if (this.superClazz != null)
/* 244 */         this.superClazz.reset(bean, context);
/* 245 */       for (Property<BeanT> p : this.properties)
/* 246 */         p.reset(bean);
/* 247 */       return true;
/*     */     } catch (AccessorException e) {
/* 249 */       context.handleError(e); }
/* 250 */     return false;
/*     */   }
/*     */   
/*     */   public String getId(BeanT bean, XMLSerializer target) throws SAXException
/*     */   {
/* 255 */     if (this.idProperty != null) {
/*     */       try {
/* 257 */         return this.idProperty.getIdValue(bean);
/*     */       } catch (AccessorException e) {
/* 259 */         target.reportError(null, e);
/*     */       }
/*     */     }
/* 262 */     return null;
/*     */   }
/*     */   
/*     */   public void serializeRoot(BeanT bean, XMLSerializer target) throws SAXException, IOException, XMLStreamException {
/* 266 */     if (this.tagName == null) {
/* 267 */       target.reportError(new ValidationEventImpl(1, Messages.UNABLE_TO_MARSHAL_NON_ELEMENT.format(new Object[] { bean.getClass().getName() }), null, null));
/*     */ 
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/*     */ 
/*     */ 
/* 275 */       target.startElement(this.tagName, bean);
/* 276 */       target.childAsSoleContent(bean, null);
/* 277 */       target.endElement();
/*     */     }
/*     */   }
/*     */   
/*     */   public void serializeBody(BeanT bean, XMLSerializer target) throws SAXException, IOException, XMLStreamException {
/* 282 */     if (this.superClazz != null)
/* 283 */       this.superClazz.serializeBody(bean, target);
/*     */     try {
/* 285 */       for (Property<BeanT> p : this.properties)
/* 286 */         p.serializeBody(bean, target, null);
/*     */     } catch (AccessorException e) {
/* 288 */       target.reportError(null, e);
/*     */     }
/*     */   }
/*     */   
/*     */   public void serializeAttributes(BeanT bean, XMLSerializer target) throws SAXException, IOException, XMLStreamException {
/*     */     try {
/* 294 */       for (AttributeProperty<BeanT> p : this.attributeProperties) {
/* 295 */         p.serializeAttributes(bean, target);
/*     */       }
/* 297 */       if (this.inheritedAttWildcard != null) {
/* 298 */         Map<QName, String> map = (Map)this.inheritedAttWildcard.get(bean);
/* 299 */         target.attWildcardAsAttributes(map, null);
/*     */       }
/*     */     } catch (AccessorException e) {
/* 302 */       target.reportError(null, e);
/*     */     }
/*     */   }
/*     */   
/*     */   public void serializeURIs(BeanT bean, XMLSerializer target) throws SAXException {
/*     */     try {
/* 308 */       for (Property<BeanT> p : this.uriProperties) {
/* 309 */         p.serializeURIs(bean, target);
/*     */       }
/* 311 */       if (this.inheritedAttWildcard != null) {
/* 312 */         Map<QName, String> map = (Map)this.inheritedAttWildcard.get(bean);
/* 313 */         target.attWildcardAsURIs(map, null);
/*     */       }
/*     */     } catch (AccessorException e) {
/* 316 */       target.reportError(null, e);
/*     */     }
/*     */   }
/*     */   
/*     */   public Loader getLoader(JAXBContextImpl context, boolean typeSubstitutionCapable) {
/* 321 */     if (this.loader == null)
/*     */     {
/*     */ 
/* 324 */       StructureLoader sl = new StructureLoader(this);
/* 325 */       this.loader = sl;
/* 326 */       if (this.ci.hasSubClasses()) {
/* 327 */         this.loaderWithTypeSubst = new XsiTypeLoader(this);
/*     */       }
/*     */       else {
/* 330 */         this.loaderWithTypeSubst = this.loader;
/*     */       }
/*     */       
/* 333 */       sl.init(context, this, this.ci.getAttributeWildcard());
/*     */     }
/* 335 */     if (typeSubstitutionCapable) {
/* 336 */       return this.loaderWithTypeSubst;
/*     */     }
/* 338 */     return this.loader;
/*     */   }
/*     */   
/*     */   public Transducer<BeanT> getTransducer() {
/* 342 */     return this.xducer;
/*     */   }
/*     */   
/* 345 */   private static final AttributeProperty[] EMPTY_PROPERTIES = new AttributeProperty[0];
/*     */   
/* 347 */   private static final Logger logger = Util.getClassLogger();
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\target\myautosupport\WEB-INF\lib\jaxb-impl-2.0.3.jar!\com\sun\xml\bind\v2\runtime\ClassBeanInfoImpl.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */