/*     */ package org.apache.avro.io.parsing;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.NoSuchElementException;
/*     */ import org.apache.avro.Schema.Field;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class Symbol
/*     */ {
/*     */   public final Kind kind;
/*     */   public final Symbol[] production;
/*     */   
/*     */   public static enum Kind
/*     */   {
/*  39 */     TERMINAL, 
/*     */     
/*  41 */     ROOT, 
/*     */     
/*  43 */     SEQUENCE, 
/*     */     
/*  45 */     REPEATER, 
/*     */     
/*  47 */     ALTERNATIVE, 
/*     */     
/*  49 */     IMPLICIT_ACTION, 
/*     */     
/*  51 */     EXPLICIT_ACTION;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     private Kind() {}
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Symbol(Kind kind)
/*     */   {
/*  78 */     this(kind, null);
/*     */   }
/*     */   
/*     */   protected Symbol(Kind kind, Symbol[] production)
/*     */   {
/*  83 */     this.production = production;
/*  84 */     this.kind = kind;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   static Symbol root(Symbol... symbols)
/*     */   {
/*  91 */     return new Root(symbols, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   static Symbol seq(Symbol... production)
/*     */   {
/*  98 */     return new Sequence(production, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   static Symbol repeat(Symbol endSymbol, Symbol... symsToRepeat)
/*     */   {
/* 106 */     return new Repeater(endSymbol, symsToRepeat, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   static Symbol alt(Symbol[] symbols, String[] labels)
/*     */   {
/* 113 */     return new Alternative(symbols, labels, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static Symbol error(String e)
/*     */   {
/* 122 */     return new ErrorAction(e, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static Symbol resolve(Symbol w, Symbol r)
/*     */   {
/* 131 */     return new ResolvingAction(w, r, null);
/*     */   }
/*     */   
/*     */   private static class Fixup {
/*     */     public final Symbol[] symbols;
/*     */     public final int pos;
/*     */     
/*     */     public Fixup(Symbol[] symbols, int pos) {
/* 139 */       this.symbols = symbols;
/* 140 */       this.pos = pos;
/*     */     }
/*     */   }
/*     */   
/*     */   public Symbol flatten(Map<Sequence, Sequence> map, Map<Sequence, List<Fixup>> map2)
/*     */   {
/* 146 */     return this;
/*     */   }
/*     */   
/*     */   public int flattenedSize() {
/* 150 */     return 1;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static void flatten(Symbol[] in, int start, Symbol[] out, int skip, Map<Sequence, Sequence> map, Map<Sequence, List<Fixup>> map2)
/*     */   {
/* 187 */     int i = start; for (int j = skip; i < in.length; i++) {
/* 188 */       Symbol s = in[i].flatten(map, map2);
/* 189 */       if ((s instanceof Sequence)) {
/* 190 */         Symbol[] p = s.production;
/* 191 */         List<Fixup> l = (List)map2.get(s);
/* 192 */         if (l == null) {
/* 193 */           System.arraycopy(p, 0, out, j, p.length);
/*     */         } else {
/* 195 */           l.add(new Fixup(out, j));
/*     */         }
/* 197 */         j += p.length;
/*     */       } else {
/* 199 */         out[(j++)] = s;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected static int flattenedSize(Symbol[] symbols, int start)
/*     */   {
/* 213 */     int result = 0;
/* 214 */     for (int i = start; i < symbols.length; i++) {
/* 215 */       if ((symbols[i] instanceof Sequence)) {
/* 216 */         Sequence s = (Sequence)symbols[i];
/* 217 */         result += s.flattenedSize();
/*     */       } else {
/* 219 */         result++;
/*     */       }
/*     */     }
/* 222 */     return result;
/*     */   }
/*     */   
/*     */   private static class Terminal extends Symbol {
/*     */     private final String printName;
/*     */     
/* 228 */     public Terminal(String printName) { super();
/* 229 */       this.printName = printName; }
/*     */     
/* 231 */     public String toString() { return this.printName; }
/*     */   }
/*     */   
/*     */ 
/*     */   public static class ImplicitAction
/*     */     extends Symbol
/*     */   {
/*     */     public final boolean isTrailing;
/*     */     
/*     */ 
/*     */     private ImplicitAction()
/*     */     {
/* 243 */       this(false);
/*     */     }
/*     */     
/*     */     private ImplicitAction(boolean isTrailing) {
/* 247 */       super();
/* 248 */       this.isTrailing = isTrailing;
/*     */     }
/*     */   }
/*     */   
/*     */   protected static class Root extends Symbol {
/*     */     private Root(Symbol... symbols) {
/* 254 */       super(makeProduction(symbols));
/* 255 */       this.production[0] = this;
/*     */     }
/*     */     
/*     */     private static Symbol[] makeProduction(Symbol[] symbols) {
/* 259 */       Symbol[] result = new Symbol[flattenedSize(symbols, 0) + 1];
/* 260 */       flatten(symbols, 0, result, 1, new HashMap(), new HashMap());
/*     */       
/*     */ 
/* 263 */       return result;
/*     */     }
/*     */   }
/*     */   
/*     */   protected static class Sequence extends Symbol implements Iterable<Symbol> {
/*     */     private Sequence(Symbol[] productions) {
/* 269 */       super(productions);
/*     */     }
/*     */     
/*     */     public Symbol get(int index) {
/* 273 */       return this.production[index];
/*     */     }
/*     */     
/*     */     public int size() {
/* 277 */       return this.production.length;
/*     */     }
/*     */     
/*     */     public Iterator<Symbol> iterator() {
/* 281 */       new Iterator() {
/* 282 */         private int pos = Symbol.Sequence.this.production.length;
/*     */         
/*     */         public boolean hasNext() {
/* 285 */           return 0 < this.pos;
/*     */         }
/*     */         
/*     */         public Symbol next() {
/* 289 */           if (0 < this.pos) {
/* 290 */             return Symbol.Sequence.this.production[(--this.pos)];
/*     */           }
/* 292 */           throw new NoSuchElementException();
/*     */         }
/*     */         
/*     */         public void remove()
/*     */         {
/* 297 */           throw new UnsupportedOperationException();
/*     */         }
/*     */       };
/*     */     }
/*     */     
/*     */     public Sequence flatten(Map<Sequence, Sequence> map, Map<Sequence, List<Symbol.Fixup>> map2)
/*     */     {
/* 304 */       Sequence result = (Sequence)map.get(this);
/* 305 */       if (result == null) {
/* 306 */         result = new Sequence(new Symbol[flattenedSize()]);
/* 307 */         map.put(this, result);
/* 308 */         List<Symbol.Fixup> l = new ArrayList();
/* 309 */         map2.put(result, l);
/*     */         
/* 311 */         flatten(this.production, 0, result.production, 0, map, map2);
/*     */         
/* 313 */         for (Symbol.Fixup f : l) {
/* 314 */           System.arraycopy(result.production, 0, f.symbols, f.pos, result.production.length);
/*     */         }
/*     */         
/* 317 */         map2.remove(result);
/*     */       }
/* 319 */       return result;
/*     */     }
/*     */     
/*     */     public final int flattenedSize()
/*     */     {
/* 324 */       return flattenedSize(this.production, 0);
/*     */     }
/*     */   }
/*     */   
/*     */   public static class Repeater extends Symbol {
/*     */     public final Symbol end;
/*     */     
/*     */     private Repeater(Symbol end, Symbol... sequenceToRepeat) {
/* 332 */       super(makeProduction(sequenceToRepeat));
/* 333 */       this.end = end;
/* 334 */       this.production[0] = this;
/*     */     }
/*     */     
/*     */     private static Symbol[] makeProduction(Symbol[] p) {
/* 338 */       Symbol[] result = new Symbol[p.length + 1];
/* 339 */       System.arraycopy(p, 0, result, 1, p.length);
/* 340 */       return result;
/*     */     }
/*     */     
/*     */ 
/*     */     public Repeater flatten(Map<Symbol.Sequence, Symbol.Sequence> map, Map<Symbol.Sequence, List<Symbol.Fixup>> map2)
/*     */     {
/* 346 */       Repeater result = new Repeater(this.end, new Symbol[flattenedSize(this.production, 1)]);
/*     */       
/* 348 */       flatten(this.production, 1, result.production, 1, map, map2);
/* 349 */       return result;
/*     */     }
/*     */   }
/*     */   
/*     */   public static class Alternative extends Symbol {
/*     */     public final Symbol[] symbols;
/*     */     public final String[] labels;
/*     */     
/*     */     private Alternative(Symbol[] symbols, String[] labels) {
/* 358 */       super();
/* 359 */       this.symbols = symbols;
/* 360 */       this.labels = labels;
/*     */     }
/*     */     
/*     */     public Symbol getSymbol(int index) {
/* 364 */       return this.symbols[index];
/*     */     }
/*     */     
/*     */     public String getLabel(int index) {
/* 368 */       return this.labels[index];
/*     */     }
/*     */     
/*     */     public int size() {
/* 372 */       return this.symbols.length;
/*     */     }
/*     */     
/*     */     public int findLabel(String label) {
/* 376 */       if (label != null) {
/* 377 */         for (int i = 0; i < this.labels.length; i++) {
/* 378 */           if (label.equals(this.labels[i])) {
/* 379 */             return i;
/*     */           }
/*     */         }
/*     */       }
/* 383 */       return -1;
/*     */     }
/*     */     
/*     */ 
/*     */     public Alternative flatten(Map<Symbol.Sequence, Symbol.Sequence> map, Map<Symbol.Sequence, List<Symbol.Fixup>> map2)
/*     */     {
/* 389 */       Symbol[] ss = new Symbol[this.symbols.length];
/* 390 */       for (int i = 0; i < ss.length; i++) {
/* 391 */         ss[i] = this.symbols[i].flatten(map, map2);
/*     */       }
/* 393 */       return new Alternative(ss, this.labels);
/*     */     }
/*     */   }
/*     */   
/*     */   public static class ErrorAction extends Symbol.ImplicitAction { public final String msg;
/*     */     
/* 399 */     private ErrorAction(String msg) { super();
/* 400 */       this.msg = msg;
/*     */     }
/*     */   }
/*     */   
/*     */   public static class IntCheckAction extends Symbol {
/*     */     public final int size;
/*     */     
/* 407 */     public IntCheckAction(int size) { super();
/* 408 */       this.size = size;
/*     */     }
/*     */   }
/*     */   
/*     */   public static class EnumAdjustAction extends Symbol.IntCheckAction {
/*     */     public final Object[] adjustments;
/*     */     
/* 415 */     public EnumAdjustAction(int rsymCount, Object[] adjustments) { super();
/* 416 */       this.adjustments = adjustments;
/*     */     }
/*     */   }
/*     */   
/* 420 */   public static class WriterUnionAction extends Symbol.ImplicitAction { public WriterUnionAction() { super(); }
/*     */   }
/*     */   
/*     */   public static class ResolvingAction extends Symbol.ImplicitAction { public final Symbol writer;
/*     */     public final Symbol reader;
/*     */     
/* 426 */     private ResolvingAction(Symbol writer, Symbol reader) { super();
/* 427 */       this.writer = writer;
/* 428 */       this.reader = reader;
/*     */     }
/*     */     
/*     */ 
/*     */     public ResolvingAction flatten(Map<Symbol.Sequence, Symbol.Sequence> map, Map<Symbol.Sequence, List<Symbol.Fixup>> map2)
/*     */     {
/* 434 */       return new ResolvingAction(this.writer.flatten(map, map2), this.reader.flatten(map, map2));
/*     */     }
/*     */   }
/*     */   
/*     */   public static class SkipAction extends Symbol.ImplicitAction
/*     */   {
/*     */     public final Symbol symToSkip;
/*     */     
/*     */     public SkipAction(Symbol symToSkip) {
/* 443 */       super(null);
/* 444 */       this.symToSkip = symToSkip;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 450 */     public SkipAction flatten(Map<Symbol.Sequence, Symbol.Sequence> map, Map<Symbol.Sequence, List<Symbol.Fixup>> map2) { return new SkipAction(this.symToSkip.flatten(map, map2)); }
/*     */   }
/*     */   
/*     */   public static class FieldAdjustAction extends Symbol.ImplicitAction {
/*     */     public final int rindex;
/*     */     public final String fname;
/*     */     
/*     */     public FieldAdjustAction(int rindex, String fname) {
/* 458 */       super();
/* 459 */       this.rindex = rindex;
/* 460 */       this.fname = fname;
/*     */     }
/*     */   }
/*     */   
/*     */   public static final class FieldOrderAction extends Symbol.ImplicitAction { public final Schema.Field[] fields;
/*     */     
/* 466 */     public FieldOrderAction(Schema.Field[] fields) { super();
/* 467 */       this.fields = fields;
/*     */     }
/*     */   }
/*     */   
/*     */   public static class DefaultStartAction extends Symbol.ImplicitAction { public final byte[] contents;
/*     */     
/* 473 */     public DefaultStartAction(byte[] contents) { super();
/* 474 */       this.contents = contents;
/*     */     }
/*     */   }
/*     */   
/*     */   public static class UnionAdjustAction extends Symbol.ImplicitAction { public final int rindex;
/*     */     public final Symbol symToParse;
/*     */     
/* 481 */     public UnionAdjustAction(int rindex, Symbol symToParse) { super();
/* 482 */       this.rindex = rindex;
/* 483 */       this.symToParse = symToParse;
/*     */     }
/*     */     
/*     */ 
/*     */     public UnionAdjustAction flatten(Map<Symbol.Sequence, Symbol.Sequence> map, Map<Symbol.Sequence, List<Symbol.Fixup>> map2)
/*     */     {
/* 489 */       return new UnionAdjustAction(this.rindex, this.symToParse.flatten(map, map2));
/*     */     }
/*     */   }
/*     */   
/*     */   public static class EnumLabelsAction extends Symbol.IntCheckAction
/*     */   {
/*     */     public final List<String> symbols;
/*     */     
/*     */     public EnumLabelsAction(List<String> symbols) {
/* 498 */       super();
/* 499 */       this.symbols = symbols;
/*     */     }
/*     */     
/*     */     public String getLabel(int n) {
/* 503 */       return (String)this.symbols.get(n);
/*     */     }
/*     */     
/*     */     public int findLabel(String l) {
/* 507 */       if (l != null) {
/* 508 */         for (int i = 0; i < this.symbols.size(); i++) {
/* 509 */           if (l.equals(this.symbols.get(i))) {
/* 510 */             return i;
/*     */           }
/*     */         }
/*     */       }
/* 514 */       return -1;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/* 521 */   public static final Symbol NULL = new Terminal("null");
/* 522 */   public static final Symbol BOOLEAN = new Terminal("boolean");
/* 523 */   public static final Symbol INT = new Terminal("int");
/* 524 */   public static final Symbol LONG = new Terminal("long");
/* 525 */   public static final Symbol FLOAT = new Terminal("float");
/* 526 */   public static final Symbol DOUBLE = new Terminal("double");
/* 527 */   public static final Symbol STRING = new Terminal("string");
/* 528 */   public static final Symbol BYTES = new Terminal("bytes");
/* 529 */   public static final Symbol FIXED = new Terminal("fixed");
/* 530 */   public static final Symbol ENUM = new Terminal("enum");
/* 531 */   public static final Symbol UNION = new Terminal("union");
/*     */   
/* 533 */   public static final Symbol ARRAY_START = new Terminal("array-start");
/* 534 */   public static final Symbol ARRAY_END = new Terminal("array-end");
/* 535 */   public static final Symbol MAP_START = new Terminal("map-start");
/* 536 */   public static final Symbol MAP_END = new Terminal("map-end");
/* 537 */   public static final Symbol ITEM_END = new Terminal("item-end");
/*     */   
/*     */ 
/* 540 */   public static final Symbol FIELD_ACTION = new Terminal("field-action");
/*     */   
/*     */ 
/* 543 */   public static final Symbol RECORD_START = new ImplicitAction(false, null);
/* 544 */   public static final Symbol RECORD_END = new ImplicitAction(true, null);
/* 545 */   public static final Symbol UNION_END = new ImplicitAction(true, null);
/* 546 */   public static final Symbol FIELD_END = new ImplicitAction(true, null);
/*     */   
/* 548 */   public static final Symbol DEFAULT_END_ACTION = new ImplicitAction(true, null);
/* 549 */   public static final Symbol MAP_KEY_MARKER = new Terminal("map-key-marker");
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\parsing\Symbol.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */