/*      */ package org.codehaus.janino.util;
/*      */ 
/*      */ import java.io.ByteArrayInputStream;
/*      */ import java.io.ByteArrayOutputStream;
/*      */ import java.io.DataInputStream;
/*      */ import java.io.DataOutputStream;
/*      */ import java.io.FilterOutputStream;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.io.OutputStream;
/*      */ import java.io.UTFDataFormatException;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Arrays;
/*      */ import java.util.Collections;
/*      */ import java.util.HashMap;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import org.codehaus.janino.Descriptor;
/*      */ import org.codehaus.janino.JaninoRuntimeException;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class ClassFile
/*      */ {
/*      */   private static final int CLASS_FILE_MAGIC = -889275714;
/*      */   public static final short MAJOR_VERSION_JDK_1_1 = 45;
/*      */   public static final short MINOR_VERSION_JDK_1_1 = 3;
/*      */   public static final short MAJOR_VERSION_JDK_1_2 = 46;
/*      */   public static final short MINOR_VERSION_JDK_1_2 = 0;
/*      */   public static final short MAJOR_VERSION_JDK_1_3 = 47;
/*      */   public static final short MINOR_VERSION_JDK_1_3 = 0;
/*      */   public static final short MAJOR_VERSION_JDK_1_4 = 48;
/*      */   public static final short MINOR_VERSION_JDK_1_4 = 0;
/*      */   public static final short MAJOR_VERSION_JDK_1_5 = 49;
/*      */   public static final short MINOR_VERSION_JDK_1_5 = 0;
/*      */   private short majorVersion;
/*      */   private short minorVersion;
/*      */   public List constantPool;
/*      */   public short accessFlags;
/*      */   public short thisClass;
/*      */   public short superclass;
/*      */   public short[] interfaces;
/*      */   public List fieldInfos;
/*      */   public List methodInfos;
/*      */   private List attributes;
/*      */   private Map constantPoolMap;
/*      */   
/*      */   public ClassFile(short accessFlags, String thisClassFD, String superclassFD, String[] interfaceFDs)
/*      */   {
/*   73 */     this.majorVersion = 45;
/*   74 */     this.minorVersion = 3;
/*      */     
/*   76 */     this.constantPool = new ArrayList();
/*   77 */     this.constantPool.add(null);
/*   78 */     this.constantPoolMap = new HashMap();
/*      */     
/*   80 */     this.accessFlags = accessFlags;
/*   81 */     this.thisClass = addConstantClassInfo(thisClassFD);
/*   82 */     this.superclass = addConstantClassInfo(superclassFD);
/*   83 */     this.interfaces = new short[interfaceFDs.length];
/*   84 */     for (int i = 0; i < interfaceFDs.length; i++) {
/*   85 */       this.interfaces[i] = addConstantClassInfo(interfaceFDs[i]);
/*      */     }
/*      */     
/*   88 */     this.fieldInfos = new ArrayList();
/*   89 */     this.methodInfos = new ArrayList();
/*   90 */     this.attributes = new ArrayList();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public void addSourceFileAttribute(String sourceFileName)
/*      */   {
/*   98 */     this.attributes.add(new SourceFileAttribute(addConstantUtf8Info("SourceFile"), addConstantUtf8Info(sourceFileName)));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public void addDeprecatedAttribute()
/*      */   {
/*  105 */     this.attributes.add(new DeprecatedAttribute(addConstantUtf8Info("Deprecated")));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public InnerClassesAttribute getInnerClassesAttribute()
/*      */   {
/*  113 */     Short ni = (Short)this.constantPoolMap.get(new ConstantUtf8Info("InnerClasses"));
/*  114 */     if (ni == null) { return null;
/*      */     }
/*  116 */     for (Iterator it = this.attributes.iterator(); it.hasNext();) {
/*  117 */       AttributeInfo ai = (AttributeInfo)it.next();
/*  118 */       if ((ai.nameIndex == ni.shortValue()) && ((ai instanceof InnerClassesAttribute))) {
/*  119 */         return (InnerClassesAttribute)ai;
/*      */       }
/*      */     }
/*  122 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void addInnerClassesAttributeEntry(ClassFile.InnerClassesAttribute.Entry e)
/*      */   {
/*  131 */     InnerClassesAttribute ica = getInnerClassesAttribute();
/*  132 */     if (ica == null) {
/*  133 */       ica = new InnerClassesAttribute(addConstantUtf8Info("InnerClasses"));
/*  134 */       this.attributes.add(ica);
/*      */     }
/*  136 */     ica.getEntries().add(e);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ClassFile(InputStream inputStream)
/*      */     throws IOException
/*      */   {
/*  152 */     DataInputStream dis = (inputStream instanceof DataInputStream) ? (DataInputStream)inputStream : new DataInputStream(inputStream);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  158 */     int magic = dis.readInt();
/*  159 */     if (magic != -889275714) { throw new ClassFormatError("Invalid magic number");
/*      */     }
/*  161 */     this.minorVersion = dis.readShort();
/*  162 */     this.majorVersion = dis.readShort();
/*  163 */     if (!isRecognizedVersion(this.majorVersion, this.minorVersion)) {
/*  164 */       throw new ClassFormatError("Unrecognized class file format version " + this.majorVersion + "/" + this.minorVersion);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  172 */     this.constantPool = new ArrayList();
/*  173 */     this.constantPoolMap = new HashMap();
/*  174 */     loadConstantPool(dis);
/*      */     
/*  176 */     this.accessFlags = dis.readShort();
/*  177 */     this.thisClass = dis.readShort();
/*  178 */     this.superclass = dis.readShort();
/*  179 */     this.interfaces = readShortArray(dis);
/*      */     
/*  181 */     this.fieldInfos = Collections.unmodifiableList(loadFields(dis));
/*  182 */     this.methodInfos = Collections.unmodifiableList(loadMethods(dis));
/*  183 */     this.attributes = Collections.unmodifiableList(loadAttributes(dis));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public String getThisClassName()
/*      */   {
/*  190 */     return getConstantClassName(this.thisClass).replace('/', '.');
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setVersion(short majorVersion, short minorVersion)
/*      */   {
/*  200 */     this.majorVersion = majorVersion;
/*  201 */     this.minorVersion = minorVersion;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public short getMajorVersion()
/*      */   {
/*  208 */     return this.majorVersion;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public short getMinorVersion()
/*      */   {
/*  215 */     return this.minorVersion;
/*      */   }
/*      */   
/*      */   public static boolean isRecognizedVersion(short majorVersion, short minorVersion) {
/*  219 */     return ((majorVersion == 45) && (minorVersion == 3)) || ((majorVersion == 46) && (minorVersion == 0)) || ((majorVersion == 47) && (minorVersion == 0)) || ((majorVersion == 48) && (minorVersion == 0)) || ((majorVersion == 49) && (minorVersion == 0));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantClassInfo(String typeFD)
/*      */   {
/*      */     String s;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  238 */     if (Descriptor.isClassOrInterfaceReference(typeFD)) {
/*  239 */       s = Descriptor.toInternalForm(typeFD);
/*      */     } else { String s;
/*  241 */       if (Descriptor.isArrayReference(typeFD)) {
/*  242 */         s = typeFD;
/*      */       }
/*      */       else
/*  245 */         throw new JaninoRuntimeException("\"" + Descriptor.toString(typeFD) + "\" is neither a class nor an array");
/*      */     }
/*      */     String s;
/*  248 */     return addToConstantPool(new ConstantClassInfo(addConstantUtf8Info(s)));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantFieldrefInfo(String classFD, String fieldName, String fieldFD)
/*      */   {
/*  262 */     return addToConstantPool(new ConstantFieldrefInfo(addConstantClassInfo(classFD), addConstantNameAndTypeInfo(fieldName, fieldFD)));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantMethodrefInfo(String classFD, String methodName, String methodMD)
/*      */   {
/*  279 */     return addToConstantPool(new ConstantMethodrefInfo(addConstantClassInfo(classFD), addConstantNameAndTypeInfo(methodName, methodMD)));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantInterfaceMethodrefInfo(String classFD, String methodName, String methodMD)
/*      */   {
/*  296 */     return addToConstantPool(new ConstantInterfaceMethodrefInfo(addConstantClassInfo(classFD), addConstantNameAndTypeInfo(methodName, methodMD)));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantStringInfo(String string)
/*      */   {
/*  309 */     return addToConstantPool(new ConstantStringInfo(addConstantUtf8Info(string)));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantIntegerInfo(int value)
/*      */   {
/*  319 */     return addToConstantPool(new ConstantIntegerInfo(value));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantFloatInfo(float value)
/*      */   {
/*  329 */     return addToConstantPool(new ConstantFloatInfo(value));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantLongInfo(long value)
/*      */   {
/*  339 */     return addToConstantPool(new ConstantLongInfo(value));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantDoubleInfo(double value)
/*      */   {
/*  349 */     return addToConstantPool(new ConstantDoubleInfo(value));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private short addConstantNameAndTypeInfo(String name, String descriptor)
/*      */   {
/*  359 */     return addToConstantPool(new ConstantNameAndTypeInfo(addConstantUtf8Info(name), addConstantUtf8Info(descriptor)));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short addConstantUtf8Info(String s)
/*      */   {
/*  372 */     return addToConstantPool(new ConstantUtf8Info(s));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private short addConstantSIFLDInfo(Object cv)
/*      */   {
/*  379 */     if ((cv instanceof String)) {
/*  380 */       return addConstantStringInfo((String)cv);
/*      */     }
/*  382 */     if (((cv instanceof Byte)) || ((cv instanceof Short)) || ((cv instanceof Integer)))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*  387 */       return addConstantIntegerInfo(((Number)cv).intValue());
/*      */     }
/*  389 */     if ((cv instanceof Boolean)) {
/*  390 */       return addConstantIntegerInfo(((Boolean)cv).booleanValue() ? 1 : 0);
/*      */     }
/*  392 */     if ((cv instanceof Character)) {
/*  393 */       return addConstantIntegerInfo(((Character)cv).charValue());
/*      */     }
/*  395 */     if ((cv instanceof Float)) {
/*  396 */       return addConstantFloatInfo(((Float)cv).floatValue());
/*      */     }
/*  398 */     if ((cv instanceof Long)) {
/*  399 */       return addConstantLongInfo(((Long)cv).longValue());
/*      */     }
/*  401 */     if ((cv instanceof Double)) {
/*  402 */       return addConstantDoubleInfo(((Double)cv).doubleValue());
/*      */     }
/*      */     
/*  405 */     throw new JaninoRuntimeException("Unexpected constant value type \"" + cv.getClass().getName() + "\"");
/*      */   }
/*      */   
/*      */   private short addToConstantPool(ConstantPoolInfo cpi)
/*      */   {
/*  410 */     Short index = (Short)this.constantPoolMap.get(cpi);
/*  411 */     if (index != null) { return index.shortValue();
/*      */     }
/*  413 */     int res = this.constantPool.size();
/*  414 */     if (res > 65535) {
/*  415 */       throw new JaninoRuntimeException("Constant pool has grown past JVM limit of 0xFFFF");
/*      */     }
/*  417 */     this.constantPool.add(cpi);
/*  418 */     if (cpi.isWide()) { this.constantPool.add(null);
/*      */     }
/*  420 */     this.constantPoolMap.put(cpi, new Short((short)res));
/*  421 */     return (short)res;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public FieldInfo addFieldInfo(short accessFlags, String fieldName, String fieldTypeFD, Object optionalConstantValue)
/*      */   {
/*  430 */     List attributes = new ArrayList();
/*  431 */     if (optionalConstantValue != null) {
/*  432 */       attributes.add(new ConstantValueAttribute(addConstantUtf8Info("ConstantValue"), addConstantSIFLDInfo(optionalConstantValue)));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  437 */     FieldInfo fi = new FieldInfo(accessFlags, addConstantUtf8Info(fieldName), addConstantUtf8Info(fieldTypeFD), attributes);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  443 */     this.fieldInfos.add(fi);
/*  444 */     return fi;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public MethodInfo addMethodInfo(short accessFlags, String methodName, String methodMD)
/*      */   {
/*  452 */     MethodInfo mi = new MethodInfo(accessFlags, addConstantUtf8Info(methodName), addConstantUtf8Info(methodMD), new ArrayList());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  458 */     this.methodInfos.add(mi);
/*  459 */     return mi;
/*      */   }
/*      */   
/*      */   public ConstantPoolInfo getConstantPoolInfo(short index) {
/*  463 */     return (ConstantPoolInfo)this.constantPool.get(0xFFFF & index);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getConstantClassName(short index)
/*      */   {
/*  471 */     ConstantClassInfo cci = (ConstantClassInfo)getConstantPoolInfo(index);
/*  472 */     ConstantUtf8Info cui = (ConstantUtf8Info)getConstantPoolInfo(cci.nameIndex);
/*  473 */     return cui.s;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getConstantUtf8(short index)
/*      */   {
/*  481 */     ConstantUtf8Info cui = (ConstantUtf8Info)getConstantPoolInfo(index);
/*  482 */     return cui.s;
/*      */   }
/*      */   
/*      */ 
/*      */   private static byte[] readLengthAndBytes(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  489 */     byte[] ba = new byte[dis.readInt()];
/*  490 */     dis.readFully(ba);
/*  491 */     return ba;
/*      */   }
/*      */   
/*      */ 
/*      */   private static short[] readShortArray(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  498 */     short count = dis.readShort();
/*  499 */     short[] result = new short[count];
/*  500 */     for (int i = 0; i < count; i++) result[i] = dis.readShort();
/*  501 */     return result;
/*      */   }
/*      */   
/*      */ 
/*      */   private void loadConstantPool(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  508 */     this.constantPool.clear();
/*  509 */     this.constantPoolMap.clear();
/*      */     
/*  511 */     short constantPoolCount = dis.readShort();
/*  512 */     this.constantPool.add(null);
/*  513 */     for (short i = 1; i < constantPoolCount; i = (short)(i + 1)) {
/*  514 */       ConstantPoolInfo cpi = ConstantPoolInfo.loadConstantPoolInfo(dis);
/*  515 */       this.constantPool.add(cpi);
/*  516 */       this.constantPoolMap.put(cpi, new Short(i));
/*  517 */       if (cpi.isWide()) {
/*  518 */         this.constantPool.add(null);
/*  519 */         i = (short)(i + 1);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private List loadFields(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  528 */     short fieldsCount = dis.readShort();
/*  529 */     List fields = new ArrayList(fieldsCount);
/*  530 */     for (int i = 0; i < fieldsCount; i++) {
/*  531 */       fields.add(new FieldInfo(dis.readShort(), dis.readShort(), dis.readShort(), loadAttributes(dis)));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  538 */     return fields;
/*      */   }
/*      */   
/*      */ 
/*      */   private List loadMethods(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  545 */     short methodsCount = dis.readShort();
/*  546 */     List methods = new ArrayList(methodsCount);
/*  547 */     for (int i = 0; i < methodsCount; i++) methods.add(loadMethodInfo(dis));
/*  548 */     return methods;
/*      */   }
/*      */   
/*      */ 
/*      */   private List loadAttributes(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/*  555 */     short attributesCount = dis.readShort();
/*  556 */     List attributes = new ArrayList(attributesCount);
/*  557 */     for (int i = 0; i < attributesCount; i++) attributes.add(loadAttribute(dis));
/*  558 */     return attributes;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void store(OutputStream os)
/*      */     throws IOException
/*      */   {
/*  572 */     DataOutputStream dos = (os instanceof DataOutputStream) ? (DataOutputStream)os : new DataOutputStream(os);
/*      */     
/*  574 */     dos.writeInt(-889275714);
/*  575 */     dos.writeShort(this.minorVersion);
/*  576 */     dos.writeShort(this.majorVersion);
/*  577 */     storeConstantPool(dos, this.constantPool);
/*  578 */     dos.writeShort(this.accessFlags);
/*  579 */     dos.writeShort(this.thisClass);
/*  580 */     dos.writeShort(this.superclass);
/*  581 */     storeShortArray(dos, this.interfaces);
/*  582 */     storeFields(dos, this.fieldInfos);
/*  583 */     storeMethods(dos, this.methodInfos);
/*  584 */     storeAttributes(dos, this.attributes);
/*      */   }
/*      */   
/*      */ 
/*      */   private static void storeConstantPool(DataOutputStream dos, List constantPool)
/*      */     throws IOException
/*      */   {
/*  591 */     dos.writeShort(constantPool.size());
/*  592 */     for (int i = 1; i < constantPool.size(); i++) {
/*  593 */       ConstantPoolInfo cpi = (ConstantPoolInfo)constantPool.get(i);
/*  594 */       if (cpi != null) {
/*  595 */         cpi.store(dos);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private static void storeShortArray(DataOutputStream dos, short[] sa)
/*      */     throws IOException
/*      */   {
/*  603 */     dos.writeShort(sa.length);
/*  604 */     for (int i = 0; i < sa.length; i++) { dos.writeShort(sa[i]);
/*      */     }
/*      */   }
/*      */   
/*      */   private static void storeFields(DataOutputStream dos, List fieldInfos)
/*      */     throws IOException
/*      */   {
/*  611 */     dos.writeShort(fieldInfos.size());
/*  612 */     for (int i = 0; i < fieldInfos.size(); i++) { ((FieldInfo)fieldInfos.get(i)).store(dos);
/*      */     }
/*      */   }
/*      */   
/*      */   private static void storeMethods(DataOutputStream dos, List methodInfos)
/*      */     throws IOException
/*      */   {
/*  619 */     dos.writeShort(methodInfos.size());
/*  620 */     for (int i = 0; i < methodInfos.size(); i++) { ((MethodInfo)methodInfos.get(i)).store(dos);
/*      */     }
/*      */   }
/*      */   
/*      */   private static void storeAttributes(DataOutputStream dos, List attributeInfos)
/*      */     throws IOException
/*      */   {
/*  627 */     dos.writeShort(attributeInfos.size());
/*  628 */     for (int i = 0; i < attributeInfos.size(); i++) { ((AttributeInfo)attributeInfos.get(i)).store(dos);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static String getSourceResourceName(String className)
/*      */   {
/*  645 */     int idx = className.lastIndexOf('.') + 1;
/*  646 */     idx = className.indexOf('$', idx);
/*  647 */     if (idx != -1) { className = className.substring(0, idx);
/*      */     }
/*      */     
/*  650 */     return className.replace('.', '/') + ".java";
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static String getClassFileResourceName(String className)
/*      */   {
/*  661 */     return className.replace('.', '/') + ".class";
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public byte[] toByteArray()
/*      */   {
/*  668 */     ByteArrayOutputStream baos = new ByteArrayOutputStream();
/*      */     try {
/*  670 */       store(baos);
/*      */     }
/*      */     catch (IOException ex) {
/*  673 */       throw new JaninoRuntimeException(ex.toString());
/*      */     }
/*  675 */     return baos.toByteArray();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static abstract class ConstantPoolInfo
/*      */   {
/*      */     public abstract void store(DataOutputStream paramDataOutputStream)
/*      */       throws IOException;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public abstract boolean isWide();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private static ConstantPoolInfo loadConstantPoolInfo(DataInputStream dis)
/*      */       throws IOException
/*      */     {
/*  710 */       byte tag = dis.readByte();
/*      */       
/*  712 */       switch (tag)
/*      */       {
/*      */       case 7: 
/*  715 */         return new ClassFile.ConstantClassInfo(dis.readShort());
/*      */       
/*      */       case 9: 
/*  718 */         return new ClassFile.ConstantFieldrefInfo(dis.readShort(), dis.readShort());
/*      */       
/*      */       case 10: 
/*  721 */         return new ClassFile.ConstantMethodrefInfo(dis.readShort(), dis.readShort());
/*      */       
/*      */       case 11: 
/*  724 */         return new ClassFile.ConstantInterfaceMethodrefInfo(dis.readShort(), dis.readShort());
/*      */       
/*      */       case 8: 
/*  727 */         return new ClassFile.ConstantStringInfo(dis.readShort());
/*      */       
/*      */       case 3: 
/*  730 */         return new ClassFile.ConstantIntegerInfo(dis.readInt());
/*      */       
/*      */       case 4: 
/*  733 */         return new ClassFile.ConstantFloatInfo(dis.readFloat());
/*      */       
/*      */       case 5: 
/*  736 */         return new ClassFile.ConstantLongInfo(dis.readLong());
/*      */       
/*      */       case 6: 
/*  739 */         return new ClassFile.ConstantDoubleInfo(dis.readDouble());
/*      */       
/*      */       case 12: 
/*  742 */         return new ClassFile.ConstantNameAndTypeInfo(dis.readShort(), dis.readShort());
/*      */       
/*      */       case 1: 
/*  745 */         return new ClassFile.ConstantUtf8Info(dis.readUTF());
/*      */       }
/*      */       
/*  748 */       throw new ClassFormatError("Invalid constant pool tag " + tag);
/*      */     }
/*      */   }
/*      */   
/*      */   public static abstract class ConstantValuePoolInfo extends ClassFile.ConstantPoolInfo {
/*      */     public abstract Object getValue(ClassFile paramClassFile);
/*      */   }
/*      */   
/*      */   public static class ConstantClassInfo extends ClassFile.ConstantPoolInfo {
/*      */     private final short nameIndex;
/*      */     
/*      */     public ConstantClassInfo(short nameIndex) {
/*  760 */       this.nameIndex = nameIndex;
/*      */     }
/*      */     
/*  763 */     public boolean isWide() { return false; }
/*      */     
/*  765 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(7);
/*  766 */       dos.writeShort(this.nameIndex);
/*      */     }
/*      */     
/*  769 */     public boolean equals(Object o) { return ((o instanceof ConstantClassInfo)) && (((ConstantClassInfo)o).nameIndex == this.nameIndex); }
/*      */     
/*  771 */     public int hashCode() { return this.nameIndex; }
/*      */   }
/*      */   
/*      */   public static class ConstantFieldrefInfo extends ClassFile.ConstantPoolInfo {
/*      */     private final short classIndex;
/*      */     private final short nameAndTypeIndex;
/*      */     
/*  778 */     public ConstantFieldrefInfo(short classIndex, short nameAndTypeIndex) { this.classIndex = classIndex;
/*  779 */       this.nameAndTypeIndex = nameAndTypeIndex;
/*      */     }
/*      */     
/*  782 */     public short getNameAndTypeIndex() { return this.nameAndTypeIndex; }
/*      */     
/*      */ 
/*  785 */     public boolean isWide() { return false; }
/*      */     
/*  787 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(9);
/*  788 */       dos.writeShort(this.classIndex);
/*  789 */       dos.writeShort(this.nameAndTypeIndex);
/*      */     }
/*      */     
/*  792 */     public boolean equals(Object o) { return ((o instanceof ConstantFieldrefInfo)) && (((ConstantFieldrefInfo)o).classIndex == this.classIndex) && (((ConstantFieldrefInfo)o).nameAndTypeIndex == this.nameAndTypeIndex); }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  798 */     public int hashCode() { return this.classIndex + (this.nameAndTypeIndex << 16); }
/*      */   }
/*      */   
/*      */   public static class ConstantMethodrefInfo extends ClassFile.ConstantPoolInfo {
/*      */     private final short classIndex;
/*      */     private final short nameAndTypeIndex;
/*      */     
/*  805 */     public ConstantMethodrefInfo(short classIndex, short nameAndTypeIndex) { this.classIndex = classIndex;
/*  806 */       this.nameAndTypeIndex = nameAndTypeIndex;
/*      */     }
/*      */     
/*  809 */     public short getNameAndTypeIndex() { return this.nameAndTypeIndex; }
/*      */     
/*      */ 
/*  812 */     public boolean isWide() { return false; }
/*      */     
/*  814 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(10);
/*  815 */       dos.writeShort(this.classIndex);
/*  816 */       dos.writeShort(this.nameAndTypeIndex);
/*      */     }
/*      */     
/*  819 */     public boolean equals(Object o) { return ((o instanceof ConstantMethodrefInfo)) && (((ConstantMethodrefInfo)o).classIndex == this.classIndex) && (((ConstantMethodrefInfo)o).nameAndTypeIndex == this.nameAndTypeIndex); }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  825 */     public int hashCode() { return this.classIndex + (this.nameAndTypeIndex << 16); }
/*      */   }
/*      */   
/*      */   public static class ConstantInterfaceMethodrefInfo extends ClassFile.ConstantPoolInfo {
/*      */     private final short classIndex;
/*      */     private final short nameAndTypeIndex;
/*      */     
/*  832 */     public ConstantInterfaceMethodrefInfo(short classIndex, short nameAndTypeIndex) { this.classIndex = classIndex;
/*  833 */       this.nameAndTypeIndex = nameAndTypeIndex;
/*      */     }
/*      */     
/*  836 */     public short getNameAndTypeIndex() { return this.nameAndTypeIndex; }
/*      */     
/*      */ 
/*  839 */     public boolean isWide() { return false; }
/*      */     
/*  841 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(11);
/*  842 */       dos.writeShort(this.classIndex);
/*  843 */       dos.writeShort(this.nameAndTypeIndex);
/*      */     }
/*      */     
/*  846 */     public boolean equals(Object o) { return ((o instanceof ConstantInterfaceMethodrefInfo)) && (((ConstantInterfaceMethodrefInfo)o).classIndex == this.classIndex) && (((ConstantInterfaceMethodrefInfo)o).nameAndTypeIndex == this.nameAndTypeIndex); }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  852 */     public int hashCode() { return this.classIndex + (this.nameAndTypeIndex << 16); }
/*      */   }
/*      */   
/*      */   static class ConstantStringInfo extends ClassFile.ConstantValuePoolInfo {
/*      */     private final short stringIndex;
/*      */     
/*  858 */     public ConstantStringInfo(short stringIndex) { this.stringIndex = stringIndex; }
/*      */     
/*      */ 
/*      */     public Object getValue(ClassFile classFile)
/*      */     {
/*  863 */       return classFile.getConstantUtf8(this.stringIndex);
/*      */     }
/*      */     
/*      */ 
/*  867 */     public boolean isWide() { return false; }
/*      */     
/*  869 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(8);
/*  870 */       dos.writeShort(this.stringIndex);
/*      */     }
/*      */     
/*  873 */     public boolean equals(Object o) { return ((o instanceof ConstantStringInfo)) && (((ConstantStringInfo)o).stringIndex == this.stringIndex); }
/*      */     
/*  875 */     public int hashCode() { return this.stringIndex; }
/*      */   }
/*      */   
/*      */   private static class ConstantIntegerInfo extends ClassFile.ConstantValuePoolInfo {
/*      */     private final int value;
/*      */     
/*  881 */     public ConstantIntegerInfo(int value) { this.value = value; }
/*      */     
/*      */     public Object getValue(ClassFile classFile)
/*      */     {
/*  885 */       return new Integer(this.value);
/*      */     }
/*      */     
/*  888 */     public boolean isWide() { return false; }
/*      */     
/*  890 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(3);
/*  891 */       dos.writeInt(this.value);
/*      */     }
/*      */     
/*  894 */     public boolean equals(Object o) { return ((o instanceof ConstantIntegerInfo)) && (((ConstantIntegerInfo)o).value == this.value); }
/*      */     
/*  896 */     public int hashCode() { return this.value; }
/*      */   }
/*      */   
/*      */   private static class ConstantFloatInfo extends ClassFile.ConstantValuePoolInfo {
/*      */     private final float value;
/*      */     
/*  902 */     public ConstantFloatInfo(float value) { this.value = value; }
/*      */     
/*      */     public Object getValue(ClassFile classFile)
/*      */     {
/*  906 */       return new Float(this.value);
/*      */     }
/*      */     
/*  909 */     public boolean isWide() { return false; }
/*      */     
/*  911 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(4);
/*  912 */       dos.writeFloat(this.value);
/*      */     }
/*      */     
/*  915 */     public boolean equals(Object o) { return ((o instanceof ConstantFloatInfo)) && (((ConstantFloatInfo)o).value == this.value); }
/*      */     
/*  917 */     public int hashCode() { return Float.floatToIntBits(this.value); }
/*      */   }
/*      */   
/*      */   private static class ConstantLongInfo extends ClassFile.ConstantValuePoolInfo {
/*      */     private final long value;
/*      */     
/*  923 */     public ConstantLongInfo(long value) { this.value = value; }
/*      */     
/*      */     public Object getValue(ClassFile classFile)
/*      */     {
/*  927 */       return new Long(this.value);
/*      */     }
/*      */     
/*  930 */     public boolean isWide() { return true; }
/*      */     
/*  932 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(5);
/*  933 */       dos.writeLong(this.value);
/*      */     }
/*      */     
/*  936 */     public boolean equals(Object o) { return ((o instanceof ConstantLongInfo)) && (((ConstantLongInfo)o).value == this.value); }
/*      */     
/*  938 */     public int hashCode() { return (int)this.value ^ (int)(this.value >> 32); }
/*      */   }
/*      */   
/*      */   private static class ConstantDoubleInfo extends ClassFile.ConstantValuePoolInfo {
/*      */     private final double value;
/*      */     
/*  944 */     public ConstantDoubleInfo(double value) { this.value = value; }
/*      */     
/*      */     public Object getValue(ClassFile classFile)
/*      */     {
/*  948 */       return new Double(this.value);
/*      */     }
/*      */     
/*  951 */     public boolean isWide() { return true; }
/*      */     
/*  953 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(6);
/*  954 */       dos.writeDouble(this.value);
/*      */     }
/*      */     
/*  957 */     public boolean equals(Object o) { return ((o instanceof ConstantDoubleInfo)) && (((ConstantDoubleInfo)o).value == this.value); }
/*      */     
/*      */     public int hashCode() {
/*  960 */       long bits = Double.doubleToLongBits(this.value);
/*  961 */       return (int)bits ^ (int)(bits >> 32);
/*      */     }
/*      */   }
/*      */   
/*      */   public static class ConstantNameAndTypeInfo extends ClassFile.ConstantPoolInfo {
/*      */     private final short nameIndex;
/*      */     private final short descriptorIndex;
/*      */     
/*  969 */     public ConstantNameAndTypeInfo(short nameIndex, short descriptorIndex) { this.nameIndex = nameIndex;
/*  970 */       this.descriptorIndex = descriptorIndex;
/*      */     }
/*      */     
/*  973 */     public short getDescriptorIndex() { return this.descriptorIndex; }
/*      */     
/*      */ 
/*  976 */     public boolean isWide() { return false; }
/*      */     
/*  978 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(12);
/*  979 */       dos.writeShort(this.nameIndex);
/*  980 */       dos.writeShort(this.descriptorIndex);
/*      */     }
/*      */     
/*  983 */     public boolean equals(Object o) { return ((o instanceof ConstantNameAndTypeInfo)) && (((ConstantNameAndTypeInfo)o).nameIndex == this.nameIndex) && (((ConstantNameAndTypeInfo)o).descriptorIndex == this.descriptorIndex); }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  989 */     public int hashCode() { return this.nameIndex + (this.descriptorIndex << 16); }
/*      */   }
/*      */   
/*      */   public static class ConstantUtf8Info extends ClassFile.ConstantPoolInfo {
/*      */     private final String s;
/*      */     
/*  995 */     public ConstantUtf8Info(String s) { if (s == null) throw new JaninoRuntimeException();
/*  996 */       this.s = s;
/*      */     }
/*      */     
/*  999 */     public String getString() { return this.s; }
/*      */     
/*      */ 
/* 1002 */     public boolean isWide() { return false; }
/*      */     
/* 1004 */     public void store(DataOutputStream dos) throws IOException { dos.writeByte(1);
/*      */       try {
/* 1006 */         dos.writeUTF(this.s);
/*      */       } catch (UTFDataFormatException e) {
/* 1008 */         throw new ClassFormatError("String constant too long to store in class file");
/*      */       }
/*      */     }
/*      */     
/* 1012 */     public boolean equals(Object o) { return ((o instanceof ConstantUtf8Info)) && (((ConstantUtf8Info)o).s.equals(this.s)); }
/*      */     
/* 1014 */     public int hashCode() { return this.s.hashCode(); }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public class MethodInfo
/*      */   {
/*      */     private final short accessFlags;
/*      */     
/*      */ 
/*      */     private final short nameIndex;
/*      */     
/*      */ 
/*      */     private final short descriptorIndex;
/*      */     
/*      */ 
/*      */     private final List attributes;
/*      */     
/*      */ 
/*      */ 
/*      */     public MethodInfo(short accessFlags, short nameIndex, short descriptorIndex, List attributes)
/*      */     {
/* 1036 */       this.accessFlags = accessFlags;
/* 1037 */       this.nameIndex = nameIndex;
/* 1038 */       this.descriptorIndex = descriptorIndex;
/* 1039 */       this.attributes = attributes;
/*      */     }
/*      */     
/* 1042 */     public ClassFile getClassFile() { return ClassFile.this; }
/*      */     
/* 1044 */     public short getAccessFlags() { return this.accessFlags; }
/* 1045 */     public short getNameIndex() { return this.nameIndex; }
/* 1046 */     public short getDescriptorIndex() { return this.descriptorIndex; }
/*      */     
/* 1048 */     public ClassFile.AttributeInfo[] getAttributes() { return (ClassFile.AttributeInfo[])this.attributes.toArray(new ClassFile.AttributeInfo[this.attributes.size()]); }
/*      */     
/*      */     public void addAttribute(ClassFile.AttributeInfo attribute)
/*      */     {
/* 1052 */       this.attributes.add(attribute);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     public void store(DataOutputStream dos)
/*      */       throws IOException
/*      */     {
/* 1060 */       dos.writeShort(this.accessFlags);
/* 1061 */       dos.writeShort(this.nameIndex);
/* 1062 */       dos.writeShort(this.descriptorIndex);
/* 1063 */       ClassFile.storeAttributes(dos, this.attributes);
/*      */     }
/*      */   }
/*      */   
/*      */   private MethodInfo loadMethodInfo(DataInputStream dis) throws IOException {
/* 1068 */     return new MethodInfo(dis.readShort(), dis.readShort(), dis.readShort(), loadAttributes(dis));
/*      */   }
/*      */   
/*      */ 
/*      */   public static class FieldInfo
/*      */   {
/*      */     private final short accessFlags;
/*      */     
/*      */     private final short nameIndex;
/*      */     
/*      */     private final short descriptorIndex;
/*      */     private final List attributes;
/*      */     
/*      */     public FieldInfo(short accessFlags, short nameIndex, short descriptorIndex, List attributes)
/*      */     {
/* 1083 */       this.accessFlags = accessFlags;
/* 1084 */       this.nameIndex = nameIndex;
/* 1085 */       this.descriptorIndex = descriptorIndex;
/* 1086 */       this.attributes = attributes;
/*      */     }
/*      */     
/* 1089 */     public short getAccessFlags() { return this.accessFlags; }
/* 1090 */     public short getNameIndex() { return this.nameIndex; }
/* 1091 */     public short getDescriptorIndex() { return this.descriptorIndex; }
/*      */     
/* 1093 */     public ClassFile.AttributeInfo[] getAttributes() { return (ClassFile.AttributeInfo[])this.attributes.toArray(new ClassFile.AttributeInfo[this.attributes.size()]); }
/*      */     
/*      */     public void addAttribute(ClassFile.AttributeInfo attribute)
/*      */     {
/* 1097 */       this.attributes.add(attribute);
/*      */     }
/*      */     
/*      */     public void store(DataOutputStream dos) throws IOException {
/* 1101 */       dos.writeShort(this.accessFlags);
/* 1102 */       dos.writeShort(this.nameIndex);
/* 1103 */       dos.writeShort(this.descriptorIndex);
/* 1104 */       ClassFile.storeAttributes(dos, this.attributes);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public static abstract class AttributeInfo
/*      */   {
/*      */     private final short nameIndex;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1118 */     public AttributeInfo(short nameIndex) { this.nameIndex = nameIndex; }
/*      */     
/*      */     public void store(DataOutputStream dos) throws IOException {
/* 1121 */       ByteArrayOutputStream baos = new ByteArrayOutputStream();
/* 1122 */       storeBody(new DataOutputStream(baos));
/*      */       
/* 1124 */       dos.writeShort(this.nameIndex);
/* 1125 */       dos.writeInt(baos.size());
/* 1126 */       baos.writeTo(dos);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     protected abstract void storeBody(DataOutputStream paramDataOutputStream)
/*      */       throws IOException;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private AttributeInfo loadAttribute(DataInputStream dis)
/*      */     throws IOException
/*      */   {
/* 1140 */     short attributeNameIndex = dis.readShort();
/* 1141 */     int attributeLength = dis.readInt();
/*      */     
/* 1143 */     byte[] ba = new byte[attributeLength];
/* 1144 */     dis.readFully(ba);
/* 1145 */     ByteArrayInputStream bais = new ByteArrayInputStream(ba);
/* 1146 */     DataInputStream bdis = new DataInputStream(bais);
/*      */     
/* 1148 */     String attributeName = getConstantUtf8(attributeNameIndex);
/*      */     AttributeInfo result;
/* 1150 */     if ("ConstantValue".equals(attributeName)) {
/* 1151 */       result = ConstantValueAttribute.loadBody(attributeNameIndex, bdis);
/*      */     } else { AttributeInfo result;
/* 1153 */       if ("Code".equals(attributeName)) {
/* 1154 */         result = CodeAttribute.loadBody(attributeNameIndex, this, bdis);
/*      */       } else { AttributeInfo result;
/* 1156 */         if ("Exceptions".equals(attributeName)) {
/* 1157 */           result = ExceptionsAttribute.loadBody(attributeNameIndex, bdis);
/*      */         } else { AttributeInfo result;
/* 1159 */           if ("InnerClasses".equals(attributeName)) {
/* 1160 */             result = InnerClassesAttribute.loadBody(attributeNameIndex, bdis);
/*      */           } else { AttributeInfo result;
/* 1162 */             if ("Synthetic".equals(attributeName)) {
/* 1163 */               result = SyntheticAttribute.loadBody(attributeNameIndex, bdis);
/*      */             } else { AttributeInfo result;
/* 1165 */               if ("SourceFile".equals(attributeName)) {
/* 1166 */                 result = SourceFileAttribute.loadBody(attributeNameIndex, bdis);
/*      */               } else { AttributeInfo result;
/* 1168 */                 if ("LineNumberTable".equals(attributeName)) {
/* 1169 */                   result = LineNumberTableAttribute.loadBody(attributeNameIndex, bdis);
/*      */                 } else { AttributeInfo result;
/* 1171 */                   if ("LocalVariableTable".equals(attributeName)) {
/* 1172 */                     result = LocalVariableTableAttribute.loadBody(attributeNameIndex, bdis);
/*      */                   } else { AttributeInfo result;
/* 1174 */                     if ("Deprecated".equals(attributeName)) {
/* 1175 */                       result = DeprecatedAttribute.loadBody(attributeNameIndex, bdis);
/*      */                     }
/*      */                     else
/* 1178 */                       new AttributeInfo(attributeNameIndex) {
/* 1179 */                         protected void storeBody(DataOutputStream dos) throws IOException { dos.write(this.val$ba); }
/*      */                       };
/*      */                   } } } } } } } }
/*      */     AttributeInfo result;
/* 1183 */     if (bais.available() > 0) {
/* 1184 */       throw new ClassFormatError(ba.length - bais.available() + " bytes of trailing garbage in body of attribute \"" + attributeName + "\"");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1192 */     return result;
/*      */   }
/*      */   
/*      */   public static class ConstantValueAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     private final short constantValueIndex;
/*      */     
/*      */     ConstantValueAttribute(short attributeNameIndex, short constantValueIndex)
/*      */     {
/* 1202 */       super();
/* 1203 */       this.constantValueIndex = constantValueIndex;
/*      */     }
/*      */     
/* 1206 */     public short getConstantValueIndex() { return this.constantValueIndex; }
/*      */     
/*      */     private static ClassFile.AttributeInfo loadBody(short attributeNameIndex, DataInputStream dis) throws IOException {
/* 1209 */       return new ConstantValueAttribute(attributeNameIndex, dis.readShort());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     protected void storeBody(DataOutputStream dos)
/*      */       throws IOException
/*      */     {
/* 1217 */       dos.writeShort(this.constantValueIndex);
/*      */     }
/*      */   }
/*      */   
/*      */   public static class ExceptionsAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     private final short[] exceptionIndexes;
/*      */     
/*      */     public ExceptionsAttribute(short attributeNameIndex, short[] exceptionIndexes)
/*      */     {
/* 1228 */       super();
/* 1229 */       this.exceptionIndexes = exceptionIndexes;
/*      */     }
/*      */     
/*      */     public short[] getExceptionIndexes() {
/* 1233 */       short[] eis = new short[this.exceptionIndexes.length];
/* 1234 */       System.arraycopy(this.exceptionIndexes, 0, eis, 0, eis.length);
/* 1235 */       return eis;
/*      */     }
/*      */     
/*      */     private static ClassFile.AttributeInfo loadBody(short attributeNameIndex, DataInputStream dis) throws IOException {
/* 1239 */       return new ExceptionsAttribute(attributeNameIndex, ClassFile.readShortArray(dis));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     protected void storeBody(DataOutputStream dos)
/*      */       throws IOException
/*      */     {
/* 1247 */       ClassFile.storeShortArray(dos, this.exceptionIndexes);
/*      */     }
/*      */   }
/*      */   
/*      */   public static class InnerClassesAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     private final List entries;
/*      */     
/*      */     InnerClassesAttribute(short attributeNameIndex)
/*      */     {
/* 1258 */       super();
/* 1259 */       this.entries = new ArrayList();
/*      */     }
/*      */     
/* 1262 */     InnerClassesAttribute(short attributeNameIndex, Entry[] entries) { super();
/* 1263 */       this.entries = new ArrayList(Arrays.asList(entries));
/*      */     }
/*      */     
/*      */     public List getEntries() {
/* 1267 */       return this.entries;
/*      */     }
/*      */     
/*      */     private static ClassFile.AttributeInfo loadBody(short attributeNameIndex, DataInputStream dis) throws IOException {
/* 1271 */       Entry[] ics = new Entry[dis.readShort()];
/* 1272 */       for (short i = 0; i < ics.length; i = (short)(i + 1)) {
/* 1273 */         ics[i] = new Entry(dis.readShort(), dis.readShort(), dis.readShort(), dis.readShort());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1280 */       return new InnerClassesAttribute(attributeNameIndex, ics);
/*      */     }
/*      */     
/*      */     protected void storeBody(DataOutputStream dos) throws IOException
/*      */     {
/* 1285 */       dos.writeShort(this.entries.size());
/* 1286 */       for (Iterator it = this.entries.iterator(); it.hasNext();) {
/* 1287 */         Entry e = (Entry)it.next();
/* 1288 */         dos.writeShort(e.innerClassInfoIndex);
/* 1289 */         dos.writeShort(e.outerClassInfoIndex);
/* 1290 */         dos.writeShort(e.innerNameIndex);
/* 1291 */         dos.writeShort(e.innerClassAccessFlags);
/*      */       }
/*      */     }
/*      */     
/*      */     public static class Entry
/*      */     {
/*      */       public final short innerClassInfoIndex;
/*      */       public final short outerClassInfoIndex;
/*      */       public final short innerNameIndex;
/*      */       public final short innerClassAccessFlags;
/*      */       
/*      */       public Entry(short innerClassInfoIndex, short outerClassInfoIndex, short innerNameIndex, short innerClassAccessFlags)
/*      */       {
/* 1304 */         this.innerClassInfoIndex = innerClassInfoIndex;
/* 1305 */         this.outerClassInfoIndex = outerClassInfoIndex;
/* 1306 */         this.innerNameIndex = innerNameIndex;
/* 1307 */         this.innerClassAccessFlags = innerClassAccessFlags;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public static class SyntheticAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     SyntheticAttribute(short attributeNameIndex)
/*      */     {
/* 1317 */       super();
/*      */     }
/*      */     
/*      */     private static ClassFile.AttributeInfo loadBody(short attributeNameIndex, DataInputStream dis) {
/* 1321 */       return new SyntheticAttribute(attributeNameIndex);
/*      */     }
/*      */     
/*      */ 
/*      */     protected void storeBody(DataOutputStream dos)
/*      */       throws IOException
/*      */     {}
/*      */   }
/*      */   
/*      */ 
/*      */   public static class SourceFileAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     private final short sourceFileIndex;
/*      */     
/*      */     public SourceFileAttribute(short attributeNameIndex, short sourceFileIndex)
/*      */     {
/* 1338 */       super();
/* 1339 */       this.sourceFileIndex = sourceFileIndex;
/*      */     }
/*      */     
/*      */     private static ClassFile.AttributeInfo loadBody(short attributeNameIndex, DataInputStream dis) throws IOException {
/* 1343 */       return new SourceFileAttribute(attributeNameIndex, dis.readShort());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     protected void storeBody(DataOutputStream dos)
/*      */       throws IOException
/*      */     {
/* 1351 */       dos.writeShort(this.sourceFileIndex);
/*      */     }
/*      */   }
/*      */   
/*      */   public static class LineNumberTableAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     private final Entry[] entries;
/*      */     
/*      */     public LineNumberTableAttribute(short attributeNameIndex, Entry[] entries)
/*      */     {
/* 1362 */       super();
/* 1363 */       this.entries = entries;
/*      */     }
/*      */     
/*      */     private static ClassFile.AttributeInfo loadBody(short attributeNameIndex, DataInputStream dis) throws IOException {
/* 1367 */       Entry[] lntes = new Entry[dis.readShort()];
/* 1368 */       for (short i = 0; i < lntes.length; i = (short)(i + 1)) {
/* 1369 */         lntes[i] = new Entry(dis.readShort(), dis.readShort());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 1374 */       return new LineNumberTableAttribute(attributeNameIndex, lntes);
/*      */     }
/*      */     
/*      */     protected void storeBody(DataOutputStream dos) throws IOException
/*      */     {
/* 1379 */       dos.writeShort(this.entries.length);
/* 1380 */       for (int i = 0; i < this.entries.length; i++) {
/* 1381 */         dos.writeShort(this.entries[i].startPC);
/* 1382 */         dos.writeShort(this.entries[i].lineNumber);
/*      */       }
/*      */     }
/*      */     
/*      */     public static class Entry { public final int startPC;
/*      */       public final int lineNumber;
/*      */       
/* 1389 */       public Entry(int startPC, int lineNumber) { this.startPC = startPC;
/* 1390 */         this.lineNumber = lineNumber;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public static class LocalVariableTableAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     private final Entry[] entries;
/*      */     
/*      */     public LocalVariableTableAttribute(short attributeNameIndex, Entry[] entries)
/*      */     {
/* 1402 */       super();
/* 1403 */       this.entries = entries;
/*      */     }
/*      */     
/*      */     private static ClassFile.AttributeInfo loadBody(short attributeNameIndex, DataInputStream dis) throws IOException {
/* 1407 */       short localVariableTableLength = dis.readShort();
/* 1408 */       Entry[] lvtes = new Entry[localVariableTableLength];
/* 1409 */       for (short i = 0; i < localVariableTableLength; i = (short)(i + 1)) {
/* 1410 */         lvtes[i] = new Entry(dis.readShort(), dis.readShort(), dis.readShort(), dis.readShort(), dis.readShort());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1418 */       return new LocalVariableTableAttribute(attributeNameIndex, lvtes);
/*      */     }
/*      */     
/*      */     protected void storeBody(DataOutputStream dos) throws IOException {
/* 1422 */       dos.writeShort(this.entries.length);
/* 1423 */       for (int i = 0; i < this.entries.length; i++) {
/* 1424 */         Entry lnte = this.entries[i];
/* 1425 */         dos.writeShort(lnte.startPC);
/* 1426 */         dos.writeShort(lnte.length);
/* 1427 */         dos.writeShort(lnte.nameIndex);
/* 1428 */         dos.writeShort(lnte.descriptorIndex);
/* 1429 */         dos.writeShort(lnte.index);
/*      */       }
/*      */     }
/*      */     
/*      */     public static class Entry
/*      */     {
/*      */       public final short startPC;
/*      */       public final short length;
/*      */       public final short nameIndex;
/*      */       public final short descriptorIndex;
/*      */       public final short index;
/*      */       
/*      */       public Entry(short startPC, short length, short nameIndex, short descriptorIndex, short index)
/*      */       {
/* 1443 */         this.startPC = startPC;
/* 1444 */         this.length = length;
/* 1445 */         this.nameIndex = nameIndex;
/* 1446 */         this.descriptorIndex = descriptorIndex;
/* 1447 */         this.index = index;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public static class DeprecatedAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     public DeprecatedAttribute(short attributeNameIndex)
/*      */     {
/* 1457 */       super();
/*      */     }
/*      */     
/*      */     private static ClassFile.AttributeInfo loadBody(short attributeNameIndex, DataInputStream dis) {
/* 1461 */       return new DeprecatedAttribute(attributeNameIndex);
/*      */     }
/*      */     
/*      */ 
/*      */     protected void storeBody(DataOutputStream dos)
/*      */       throws IOException
/*      */     {}
/*      */   }
/*      */   
/*      */ 
/*      */   private final byte[] val$ba;
/*      */   
/*      */   private static class CodeAttribute
/*      */     extends ClassFile.AttributeInfo
/*      */   {
/*      */     private final short maxStack;
/*      */     
/*      */     private final short maxLocals;
/*      */     
/*      */     private final byte[] code;
/*      */     
/*      */     private final ExceptionTableEntry[] exceptionTableEntries;
/*      */     private final ClassFile.AttributeInfo[] attributes;
/*      */     
/*      */     CodeAttribute(short attributeNameIndex, short maxStack, short maxLocals, byte[] code, ExceptionTableEntry[] exceptionTableEntries, ClassFile.AttributeInfo[] attributes)
/*      */     {
/* 1487 */       super();
/* 1488 */       this.maxStack = maxStack;
/* 1489 */       this.maxLocals = maxLocals;
/* 1490 */       this.code = code;
/* 1491 */       this.exceptionTableEntries = exceptionTableEntries;
/* 1492 */       this.attributes = attributes;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     public static ClassFile.AttributeInfo loadBody(short attributeNameIndex, ClassFile classFile, DataInputStream dis)
/*      */       throws IOException
/*      */     {
/* 1500 */       short maxStack = dis.readShort();
/* 1501 */       short maxLocals = dis.readShort();
/* 1502 */       byte[] code = ClassFile.readLengthAndBytes(dis);
/*      */       
/* 1504 */       ExceptionTableEntry[] etes = new ExceptionTableEntry[dis.readShort()];
/* 1505 */       for (int i = 0; i < etes.length; i++) {
/* 1506 */         etes[i] = new ExceptionTableEntry(dis.readShort(), dis.readShort(), dis.readShort(), dis.readShort());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1514 */       ClassFile.AttributeInfo[] attributes = new ClassFile.AttributeInfo[dis.readShort()];
/* 1515 */       for (int i = 0; i < attributes.length; i++) {
/* 1516 */         attributes[i] = classFile.loadAttribute(dis);
/*      */       }
/*      */       
/* 1519 */       return new CodeAttribute(attributeNameIndex, maxStack, maxLocals, code, etes, attributes);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     protected void storeBody(DataOutputStream dos)
/*      */       throws IOException
/*      */     {
/* 1530 */       dos.writeShort(this.maxStack);
/* 1531 */       dos.writeShort(this.maxLocals);
/* 1532 */       dos.writeInt(this.code.length);
/* 1533 */       dos.write(this.code);
/* 1534 */       dos.writeShort(this.exceptionTableEntries.length);
/* 1535 */       for (int i = 0; i < this.exceptionTableEntries.length; i++) {
/* 1536 */         ExceptionTableEntry ete = this.exceptionTableEntries[i];
/* 1537 */         dos.writeShort(ete.startPC);
/* 1538 */         dos.writeShort(ete.endPC);
/* 1539 */         dos.writeShort(ete.handlerPC);
/* 1540 */         dos.writeShort(ete.catchType);
/*      */       }
/* 1542 */       dos.writeShort(this.attributes.length);
/* 1543 */       for (int i = 0; i < this.attributes.length; i++) {
/* 1544 */         this.attributes[i].store(dos);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */     private static class ExceptionTableEntry
/*      */     {
/*      */       final short startPC;
/*      */       
/*      */       final short endPC;
/*      */       
/*      */       final short handlerPC;
/*      */       
/*      */       final short catchType;
/*      */       
/*      */       public ExceptionTableEntry(short startPC, short endPC, short handlerPC, short catchType)
/*      */       {
/* 1561 */         this.startPC = startPC;
/* 1562 */         this.endPC = endPC;
/* 1563 */         this.handlerPC = handlerPC;
/* 1564 */         this.catchType = catchType;
/*      */       }
/*      */     }
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\util\ClassFile.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */