/*     */ package org.hibernate.annotations.common.util;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.Iterator;
/*     */ import java.util.StringTokenizer;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class StringHelper
/*     */ {
/*     */   private static final int ALIAS_TRUNCATE_LENGTH = 10;
/*     */   public static final String WHITESPACE = " \n\r\f\t";
/*     */   
/*     */   public static int lastIndexOfLetter(String string)
/*     */   {
/*  29 */     for (int i = 0; i < string.length(); i++) {
/*  30 */       char character = string.charAt(i);
/*  31 */       if (!Character.isLetter(character)) return i - 1;
/*     */     }
/*  33 */     return string.length() - 1;
/*     */   }
/*     */   
/*     */   public static String join(String seperator, String[] strings) {
/*  37 */     int length = strings.length;
/*  38 */     if (length == 0) return "";
/*  39 */     StringBuffer buf = new StringBuffer(length * strings[0].length()).append(strings[0]);
/*     */     
/*  41 */     for (int i = 1; i < length; i++) {
/*  42 */       buf.append(seperator).append(strings[i]);
/*     */     }
/*  44 */     return buf.toString();
/*     */   }
/*     */   
/*     */   public static String join(String seperator, Iterator objects) {
/*  48 */     StringBuffer buf = new StringBuffer();
/*  49 */     if (objects.hasNext()) buf.append(objects.next());
/*  50 */     while (objects.hasNext()) {
/*  51 */       buf.append(seperator).append(objects.next());
/*     */     }
/*  53 */     return buf.toString();
/*     */   }
/*     */   
/*     */   public static String[] add(String[] x, String sep, String[] y) {
/*  57 */     String[] result = new String[x.length];
/*  58 */     for (int i = 0; i < x.length; i++) {
/*  59 */       result[i] = (x[i] + sep + y[i]);
/*     */     }
/*  61 */     return result;
/*     */   }
/*     */   
/*     */   public static String repeat(String string, int times) {
/*  65 */     StringBuffer buf = new StringBuffer(string.length() * times);
/*  66 */     for (int i = 0; i < times; i++) buf.append(string);
/*  67 */     return buf.toString();
/*     */   }
/*     */   
/*     */   public static String replace(String template, String placeholder, String replacement)
/*     */   {
/*  72 */     return replace(template, placeholder, replacement, false);
/*     */   }
/*     */   
/*     */   public static String[] replace(String[] templates, String placeholder, String replacement) {
/*  76 */     String[] result = new String[templates.length];
/*  77 */     for (int i = 0; i < templates.length; i++) {
/*  78 */       result[i] = replace(templates[i], placeholder, replacement);
/*     */     }
/*  80 */     return result;
/*     */   }
/*     */   
/*     */   public static String replace(String template, String placeholder, String replacement, boolean wholeWords) {
/*  84 */     int loc = template == null ? -1 : template.indexOf(placeholder);
/*  85 */     if (loc < 0) {
/*  86 */       return template;
/*     */     }
/*     */     
/*  89 */     boolean actuallyReplace = (!wholeWords) || (loc + placeholder.length() == template.length()) || (!Character.isJavaIdentifierPart(template.charAt(loc + placeholder.length())));
/*     */     
/*     */ 
/*  92 */     String actualReplacement = actuallyReplace ? replacement : placeholder;
/*  93 */     return template.substring(0, loc) + actualReplacement + replace(template.substring(loc + placeholder.length()), placeholder, replacement, wholeWords);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String replaceOnce(String template, String placeholder, String replacement)
/*     */   {
/* 104 */     int loc = template == null ? -1 : template.indexOf(placeholder);
/* 105 */     if (loc < 0) {
/* 106 */       return template;
/*     */     }
/*     */     
/* 109 */     return template.substring(0, loc) + replacement + template.substring(loc + placeholder.length());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String[] split(String seperators, String list)
/*     */   {
/* 118 */     return split(seperators, list, false);
/*     */   }
/*     */   
/*     */   public static String[] split(String seperators, String list, boolean include) {
/* 122 */     StringTokenizer tokens = new StringTokenizer(list, seperators, include);
/* 123 */     String[] result = new String[tokens.countTokens()];
/* 124 */     int i = 0;
/* 125 */     while (tokens.hasMoreTokens()) {
/* 126 */       result[(i++)] = tokens.nextToken();
/*     */     }
/* 128 */     return result;
/*     */   }
/*     */   
/*     */   public static String unqualify(String qualifiedName) {
/* 132 */     int loc = qualifiedName.lastIndexOf(".");
/* 133 */     return loc < 0 ? qualifiedName : qualifiedName.substring(qualifiedName.lastIndexOf(".") + 1);
/*     */   }
/*     */   
/*     */   public static String qualifier(String qualifiedName) {
/* 137 */     int loc = qualifiedName.lastIndexOf(".");
/* 138 */     return loc < 0 ? "" : qualifiedName.substring(0, loc);
/*     */   }
/*     */   
/*     */   public static String[] suffix(String[] columns, String suffix) {
/* 142 */     if (suffix == null) return columns;
/* 143 */     String[] qualified = new String[columns.length];
/* 144 */     for (int i = 0; i < columns.length; i++) {
/* 145 */       qualified[i] = suffix(columns[i], suffix);
/*     */     }
/* 147 */     return qualified;
/*     */   }
/*     */   
/*     */   private static String suffix(String name, String suffix) {
/* 151 */     return name + suffix;
/*     */   }
/*     */   
/*     */   public static String root(String qualifiedName) {
/* 155 */     int loc = qualifiedName.indexOf(".");
/* 156 */     return loc < 0 ? qualifiedName : qualifiedName.substring(0, loc);
/*     */   }
/*     */   
/*     */   public static String unroot(String qualifiedName) {
/* 160 */     int loc = qualifiedName.indexOf(".");
/* 161 */     return loc < 0 ? qualifiedName : qualifiedName.substring(loc + 1, qualifiedName.length());
/*     */   }
/*     */   
/*     */   public static boolean booleanValue(String tfString) {
/* 165 */     String trimmed = tfString.trim().toLowerCase();
/* 166 */     return (trimmed.equals("true")) || (trimmed.equals("t"));
/*     */   }
/*     */   
/*     */   public static String toString(Object[] array) {
/* 170 */     int len = array.length;
/* 171 */     if (len == 0) return "";
/* 172 */     StringBuffer buf = new StringBuffer(len * 12);
/* 173 */     for (int i = 0; i < len - 1; i++) {
/* 174 */       buf.append(array[i]).append(", ");
/*     */     }
/* 176 */     return (String)array[(len - 1)];
/*     */   }
/*     */   
/*     */   public static String[] multiply(String string, Iterator placeholders, Iterator replacements) {
/* 180 */     String[] result = { string };
/* 181 */     while (placeholders.hasNext()) {
/* 182 */       result = multiply(result, (String)placeholders.next(), (String[])replacements.next());
/*     */     }
/* 184 */     return result;
/*     */   }
/*     */   
/*     */   private static String[] multiply(String[] strings, String placeholder, String[] replacements) {
/* 188 */     String[] results = new String[replacements.length * strings.length];
/* 189 */     int n = 0;
/* 190 */     for (int i = 0; i < replacements.length; i++) {
/* 191 */       for (int j = 0; j < strings.length; j++) {
/* 192 */         results[(n++)] = replaceOnce(strings[j], placeholder, replacements[i]);
/*     */       }
/*     */     }
/* 195 */     return results;
/*     */   }
/*     */   
/*     */   public static int countUnquoted(String string, char character) {
/* 199 */     if ('\'' == character) {
/* 200 */       throw new IllegalArgumentException("Unquoted count of quotes is invalid");
/*     */     }
/* 202 */     if (string == null) {
/* 203 */       return 0;
/*     */     }
/*     */     
/*     */ 
/* 207 */     int count = 0;
/* 208 */     int stringLength = string.length();
/* 209 */     boolean inQuote = false;
/* 210 */     for (int indx = 0; indx < stringLength; indx++) {
/* 211 */       char c = string.charAt(indx);
/* 212 */       if (inQuote) {
/* 213 */         if ('\'' == c) {
/* 214 */           inQuote = false;
/*     */         }
/*     */       }
/* 217 */       else if ('\'' == c) {
/* 218 */         inQuote = true;
/*     */       }
/* 220 */       else if (c == character) {
/* 221 */         count++;
/*     */       }
/*     */     }
/* 224 */     return count;
/*     */   }
/*     */   
/*     */   public static int[] locateUnquoted(String string, char character) {
/* 228 */     if ('\'' == character) {
/* 229 */       throw new IllegalArgumentException("Unquoted count of quotes is invalid");
/*     */     }
/* 231 */     if (string == null) {
/* 232 */       return new int[0];
/*     */     }
/*     */     
/* 235 */     ArrayList locations = new ArrayList(20);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 240 */     int stringLength = string.length();
/* 241 */     boolean inQuote = false;
/* 242 */     for (int indx = 0; indx < stringLength; indx++) {
/* 243 */       char c = string.charAt(indx);
/* 244 */       if (inQuote) {
/* 245 */         if ('\'' == c) {
/* 246 */           inQuote = false;
/*     */         }
/*     */       }
/* 249 */       else if ('\'' == c) {
/* 250 */         inQuote = true;
/*     */       }
/* 252 */       else if (c == character) {
/* 253 */         locations.add(new Integer(indx));
/*     */       }
/*     */     }
/* 256 */     return toIntArray(locations);
/*     */   }
/*     */   
/*     */   public static boolean isNotEmpty(String string) {
/* 260 */     return (string != null) && (string.length() > 0);
/*     */   }
/*     */   
/*     */   public static boolean isEmpty(String string) {
/* 264 */     return (string == null) || (string.length() == 0);
/*     */   }
/*     */   
/*     */   public static String qualify(String prefix, String name) {
/* 268 */     if ((name == null) || (prefix == null)) {
/* 269 */       throw new NullPointerException();
/*     */     }
/* 271 */     return prefix.length() + name.length() + 1 + prefix + '.' + name;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String[] qualify(String prefix, String[] names)
/*     */   {
/* 279 */     if (prefix == null) return names;
/* 280 */     int len = names.length;
/* 281 */     String[] qualified = new String[len];
/* 282 */     for (int i = 0; i < len; i++) {
/* 283 */       qualified[i] = qualify(prefix, names[i]);
/*     */     }
/* 285 */     return qualified;
/*     */   }
/*     */   
/*     */   public static int firstIndexOfChar(String sqlString, String string, int startindex) {
/* 289 */     int matchAt = -1;
/* 290 */     for (int i = 0; i < string.length(); i++) {
/* 291 */       int curMatch = sqlString.indexOf(string.charAt(i), startindex);
/* 292 */       if (curMatch >= 0) {
/* 293 */         if (matchAt == -1) {
/* 294 */           matchAt = curMatch;
/*     */         }
/*     */         else {
/* 297 */           matchAt = Math.min(matchAt, curMatch);
/*     */         }
/*     */       }
/*     */     }
/* 301 */     return matchAt;
/*     */   }
/*     */   
/*     */   public static String truncate(String string, int length) {
/* 305 */     if (string.length() <= length) {
/* 306 */       return string;
/*     */     }
/*     */     
/* 309 */     return string.substring(0, length);
/*     */   }
/*     */   
/*     */   public static String generateAlias(String description)
/*     */   {
/* 314 */     return generateAliasRoot(description) + '_';
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String generateAlias(String description, int unique)
/*     */   {
/* 324 */     return generateAliasRoot(description) + Integer.toString(unique) + '_';
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static String generateAliasRoot(String description)
/*     */   {
/* 338 */     String result = truncate(unqualifyEntityName(description), 10).toLowerCase().replace('/', '_').replace('$', '_');
/*     */     
/*     */ 
/*     */ 
/* 342 */     result = cleanAlias(result);
/* 343 */     if (Character.isDigit(result.charAt(result.length() - 1))) {
/* 344 */       return result + "x";
/*     */     }
/*     */     
/* 347 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static String cleanAlias(String alias)
/*     */   {
/* 359 */     char[] chars = alias.toCharArray();
/*     */     
/* 361 */     if (!Character.isLetter(chars[0])) {
/* 362 */       for (int i = 1; i < chars.length; i++)
/*     */       {
/*     */ 
/* 365 */         if (Character.isLetter(chars[i])) {
/* 366 */           return alias.substring(i);
/*     */         }
/*     */       }
/*     */     }
/* 370 */     return alias;
/*     */   }
/*     */   
/*     */   public static String unqualifyEntityName(String entityName) {
/* 374 */     String result = unqualify(entityName);
/* 375 */     int slashPos = result.indexOf('/');
/* 376 */     if (slashPos > 0) {
/* 377 */       result = result.substring(0, slashPos - 1);
/*     */     }
/* 379 */     return result;
/*     */   }
/*     */   
/*     */   public static String toUpperCase(String str) {
/* 383 */     return str == null ? null : str.toUpperCase();
/*     */   }
/*     */   
/*     */   public static String toLowerCase(String str) {
/* 387 */     return str == null ? null : str.toLowerCase();
/*     */   }
/*     */   
/*     */   public static String moveAndToBeginning(String filter) {
/* 391 */     if (filter.trim().length() > 0) {
/* 392 */       filter = filter + " and ";
/* 393 */       if (filter.startsWith(" and ")) filter = filter.substring(4);
/*     */     }
/* 395 */     return filter;
/*     */   }
/*     */   
/*     */   public static int[] toIntArray(Collection coll)
/*     */   {
/* 400 */     Iterator iter = coll.iterator();
/* 401 */     int[] arr = new int[coll.size()];
/* 402 */     int i = 0;
/* 403 */     while (iter.hasNext()) {
/* 404 */       arr[(i++)] = ((Integer)iter.next()).intValue();
/*     */     }
/* 406 */     return arr;
/*     */   }
/*     */   
/*     */   public static boolean isQuoted(String name) {
/* 410 */     return (name != null) && (name.length() != 0) && (name.charAt(0) == '`');
/*     */   }
/*     */   
/*     */   public static String quote(String name) {
/* 414 */     if ((name == null) || (name.length() == 0) || (isQuoted(name))) {
/* 415 */       return name;
/*     */     }
/*     */     
/* 418 */     return name.length() + 2 + '`' + name + '`';
/*     */   }
/*     */   
/*     */   public static String unquote(String name)
/*     */   {
/* 423 */     if (isQuoted(name)) {
/* 424 */       return name.substring(1, name.length() - 1);
/*     */     }
/*     */     
/* 427 */     return name;
/*     */   }
/*     */ }


/* Location:              /Users/anandkum/Downloads/hibernate-commons-annotations-3.0.0.ga.jar!/org/hibernate/annotations/common/util/StringHelper.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */