/*     */ package org.xerial.snappy;
/*     */ 
/*     */ import java.io.BufferedInputStream;
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.File;
/*     */ import java.io.FileInputStream;
/*     */ import java.io.FileOutputStream;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.PrintStream;
/*     */ import java.lang.reflect.InvocationTargetException;
/*     */ import java.lang.reflect.Method;
/*     */ import java.net.URL;
/*     */ import java.security.DigestInputStream;
/*     */ import java.security.MessageDigest;
/*     */ import java.security.NoSuchAlgorithmException;
/*     */ import java.security.ProtectionDomain;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Enumeration;
/*     */ import java.util.List;
/*     */ import java.util.Properties;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SnappyLoader
/*     */ {
/*     */   public static final String SNAPPY_SYSTEM_PROPERTIES_FILE = "org-xerial-snappy.properties";
/*     */   public static final String KEY_SNAPPY_LIB_PATH = "org.xerial.snappy.lib.path";
/*     */   public static final String KEY_SNAPPY_LIB_NAME = "org.xerial.snappy.lib.name";
/*     */   public static final String KEY_SNAPPY_TEMPDIR = "org.xerial.snappy.tempdir";
/*     */   public static final String KEY_SNAPPY_USE_SYSTEMLIB = "org.xerial.snappy.use.systemlib";
/*     */   public static final String KEY_SNAPPY_DISABLE_BUNDLED_LIBS = "org.xerial.snappy.disable.bundled.libs";
/*  96 */   private static boolean isLoaded = false;
/*  97 */   private static Object api = null;
/*     */   
/*     */ 
/*     */ 
/*     */   private static void loadSnappySystemProperties()
/*     */   {
/*     */     try
/*     */     {
/* 105 */       InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream("org-xerial-snappy.properties");
/*     */       
/*     */ 
/* 108 */       if (is == null) {
/* 109 */         return;
/*     */       }
/*     */       
/* 112 */       Properties props = new Properties();
/* 113 */       props.load(is);
/* 114 */       is.close();
/* 115 */       Enumeration<?> names = props.propertyNames();
/* 116 */       while (names.hasMoreElements()) {
/* 117 */         String name = (String)names.nextElement();
/* 118 */         if ((name.startsWith("org.xerial.snappy.")) && 
/* 119 */           (System.getProperty(name) == null)) {
/* 120 */           System.setProperty(name, props.getProperty(name));
/*     */         }
/*     */       }
/*     */     }
/*     */     catch (Throwable ex)
/*     */     {
/* 126 */       System.err.println("Could not load 'org-xerial-snappy.properties' from classpath: " + ex.toString());
/*     */     }
/*     */   }
/*     */   
/*     */   static
/*     */   {
/* 132 */     loadSnappySystemProperties();
/*     */   }
/*     */   
/*     */   private static ClassLoader getRootClassLoader() {
/* 136 */     ClassLoader cl = Thread.currentThread().getContextClassLoader();
/* 137 */     while (cl.getParent() != null) {
/* 138 */       cl = cl.getParent();
/*     */     }
/* 140 */     return cl;
/*     */   }
/*     */   
/*     */   private static byte[] getByteCode(String resourcePath) throws IOException
/*     */   {
/* 145 */     InputStream in = SnappyLoader.class.getResourceAsStream(resourcePath);
/* 146 */     if (in == null)
/* 147 */       throw new IOException(resourcePath + " is not found");
/* 148 */     byte[] buf = new byte['Ѐ'];
/* 149 */     ByteArrayOutputStream byteCodeBuf = new ByteArrayOutputStream();
/* 150 */     int readLength; while ((readLength = in.read(buf)) != -1) {
/* 151 */       byteCodeBuf.write(buf, 0, readLength);
/*     */     }
/* 153 */     in.close();
/*     */     
/* 155 */     return byteCodeBuf.toByteArray();
/*     */   }
/*     */   
/*     */   public static boolean isNativeLibraryLoaded() {
/* 159 */     return isLoaded;
/*     */   }
/*     */   
/*     */   private static boolean hasInjectedNativeLoader() {
/*     */     try {
/* 164 */       String nativeLoaderClassName = "org.xerial.snappy.SnappyNativeLoader";
/* 165 */       Class<?> c = Class.forName("org.xerial.snappy.SnappyNativeLoader");
/*     */       
/* 167 */       return true;
/*     */     }
/*     */     catch (ClassNotFoundException e) {}
/*     */     
/* 171 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static synchronized Object load()
/*     */   {
/* 211 */     if (api != null) {
/* 212 */       return api;
/*     */     }
/*     */     try {
/* 215 */       if (!hasInjectedNativeLoader())
/*     */       {
/* 217 */         Class<?> nativeLoader = injectSnappyNativeLoader();
/*     */         
/* 219 */         loadNativeLibrary(nativeLoader);
/*     */       }
/*     */       
/* 222 */       isLoaded = true;
/*     */       
/* 224 */       Object nativeCode = Class.forName("org.xerial.snappy.SnappyNative").newInstance();
/* 225 */       api = nativeCode;
/*     */     }
/*     */     catch (Exception e) {
/* 228 */       e.printStackTrace();
/* 229 */       throw new SnappyError(SnappyErrorCode.FAILED_TO_LOAD_NATIVE_LIBRARY, e.getMessage());
/*     */     }
/*     */     
/* 232 */     return api;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static Class<?> injectSnappyNativeLoader()
/*     */   {
/*     */     try
/*     */     {
/* 245 */       String nativeLoaderClassName = "org.xerial.snappy.SnappyNativeLoader";
/* 246 */       ClassLoader rootClassLoader = getRootClassLoader();
/*     */       
/* 248 */       byte[] byteCode = getByteCode("/org/xerial/snappy/SnappyNativeLoader.bytecode");
/*     */       
/* 250 */       String[] classesToPreload = { "org.xerial.snappy.SnappyNativeAPI", "org.xerial.snappy.SnappyNative", "org.xerial.snappy.SnappyErrorCode" };
/*     */       
/* 252 */       List<byte[]> preloadClassByteCode = new ArrayList(classesToPreload.length);
/* 253 */       for (String each : classesToPreload) {
/* 254 */         preloadClassByteCode.add(getByteCode(String.format("/%s.class", new Object[] { each.replaceAll("\\.", "/") })));
/*     */       }
/*     */       
/*     */ 
/* 258 */       Class<?> classLoader = Class.forName("java.lang.ClassLoader");
/* 259 */       Method defineClass = classLoader.getDeclaredMethod("defineClass", new Class[] { String.class, byte[].class, Integer.TYPE, Integer.TYPE, ProtectionDomain.class });
/*     */       
/*     */ 
/* 262 */       ProtectionDomain pd = System.class.getProtectionDomain();
/*     */       
/*     */ 
/* 265 */       defineClass.setAccessible(true);
/*     */       try
/*     */       {
/* 268 */         defineClass.invoke(rootClassLoader, new Object[] { "org.xerial.snappy.SnappyNativeLoader", byteCode, Integer.valueOf(0), Integer.valueOf(byteCode.length), pd });
/*     */         
/*     */ 
/* 271 */         for (int i = 0; i < classesToPreload.length; i++) {
/* 272 */           byte[] b = (byte[])preloadClassByteCode.get(i);
/* 273 */           defineClass.invoke(rootClassLoader, new Object[] { classesToPreload[i], b, Integer.valueOf(0), Integer.valueOf(b.length), pd });
/*     */         }
/*     */       }
/*     */       finally
/*     */       {
/* 278 */         defineClass.setAccessible(false);
/*     */       }
/*     */       
/*     */ 
/* 282 */       return rootClassLoader.loadClass("org.xerial.snappy.SnappyNativeLoader");
/*     */     }
/*     */     catch (Exception e)
/*     */     {
/* 286 */       e.printStackTrace(System.err);
/* 287 */       throw new SnappyError(SnappyErrorCode.FAILED_TO_LOAD_NATIVE_LIBRARY, e.getMessage());
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static void loadNativeLibrary(Class<?> loaderClass)
/*     */     throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException
/*     */   {
/* 305 */     if (loaderClass == null) {
/* 306 */       throw new SnappyError(SnappyErrorCode.FAILED_TO_LOAD_NATIVE_LIBRARY, "missing snappy native loader class");
/*     */     }
/* 308 */     File nativeLib = findNativeLibrary();
/* 309 */     if (nativeLib != null)
/*     */     {
/* 311 */       Method loadMethod = loaderClass.getDeclaredMethod("load", new Class[] { String.class });
/* 312 */       loadMethod.invoke(null, new Object[] { nativeLib.getAbsolutePath() });
/*     */     }
/*     */     else
/*     */     {
/* 316 */       Method loadMethod = loaderClass.getDeclaredMethod("loadLibrary", new Class[] { String.class });
/* 317 */       loadMethod.invoke(null, new Object[] { "snappyjava" });
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static String md5sum(InputStream input)
/*     */     throws IOException
/*     */   {
/* 330 */     BufferedInputStream in = new BufferedInputStream(input);
/*     */     try {
/* 332 */       MessageDigest digest = MessageDigest.getInstance("MD5");
/* 333 */       DigestInputStream digestInputStream = new DigestInputStream(in, digest);
/* 334 */       while (digestInputStream.read() >= 0) {}
/*     */       
/*     */ 
/* 337 */       ByteArrayOutputStream md5out = new ByteArrayOutputStream();
/* 338 */       md5out.write(digest.digest());
/* 339 */       return md5out.toString();
/*     */     }
/*     */     catch (NoSuchAlgorithmException e) {
/* 342 */       throw new IllegalStateException("MD5 algorithm is not available: " + e);
/*     */     }
/*     */     finally {
/* 345 */       in.close();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static File extractLibraryFile(String libFolderForCurrentOS, String libraryFileName, String targetFolder)
/*     */   {
/* 358 */     String nativeLibraryFilePath = libFolderForCurrentOS + "/" + libraryFileName;
/* 359 */     String prefix = "snappy-" + getVersion() + "-";
/* 360 */     String extractedLibFileName = prefix + libraryFileName;
/* 361 */     File extractedLibFile = new File(targetFolder, extractedLibFileName);
/*     */     try
/*     */     {
/* 364 */       if (extractedLibFile.exists())
/*     */       {
/* 366 */         String md5sum1 = md5sum(SnappyLoader.class.getResourceAsStream(nativeLibraryFilePath));
/* 367 */         String md5sum2 = md5sum(new FileInputStream(extractedLibFile));
/*     */         
/* 369 */         if (md5sum1.equals(md5sum2)) {
/* 370 */           return new File(targetFolder, extractedLibFileName);
/*     */         }
/*     */         
/*     */ 
/* 374 */         boolean deletionSucceeded = extractedLibFile.delete();
/* 375 */         if (!deletionSucceeded) {
/* 376 */           throw new IOException("failed to remove existing native library file: " + extractedLibFile.getAbsolutePath());
/*     */         }
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 383 */       InputStream reader = SnappyLoader.class.getResourceAsStream(nativeLibraryFilePath);
/* 384 */       FileOutputStream writer = new FileOutputStream(extractedLibFile);
/* 385 */       byte[] buffer = new byte[' '];
/* 386 */       int bytesRead = 0;
/* 387 */       while ((bytesRead = reader.read(buffer)) != -1) {
/* 388 */         writer.write(buffer, 0, bytesRead);
/*     */       }
/*     */       
/* 391 */       writer.close();
/* 392 */       reader.close();
/*     */       
/*     */ 
/* 395 */       if (!System.getProperty("os.name").contains("Windows")) {
/*     */         try {
/* 397 */           Runtime.getRuntime().exec(new String[] { "chmod", "755", extractedLibFile.getAbsolutePath() }).waitFor();
/*     */         }
/*     */         catch (Throwable e) {}
/*     */       }
/*     */       
/*     */ 
/* 403 */       return new File(targetFolder, extractedLibFileName);
/*     */     }
/*     */     catch (IOException e) {
/* 406 */       e.printStackTrace(System.err); }
/* 407 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */   static File findNativeLibrary()
/*     */   {
/* 413 */     boolean useSystemLib = Boolean.parseBoolean(System.getProperty("org.xerial.snappy.use.systemlib", "false"));
/* 414 */     if (useSystemLib) {
/* 415 */       return null;
/*     */     }
/* 417 */     boolean disabledBundledLibs = Boolean.parseBoolean(System.getProperty("org.xerial.snappy.disable.bundled.libs", "false"));
/*     */     
/* 419 */     if (disabledBundledLibs) {
/* 420 */       return null;
/*     */     }
/*     */     
/* 423 */     String snappyNativeLibraryPath = System.getProperty("org.xerial.snappy.lib.path");
/* 424 */     String snappyNativeLibraryName = System.getProperty("org.xerial.snappy.lib.name");
/*     */     
/*     */ 
/* 427 */     if (snappyNativeLibraryName == null) {
/* 428 */       snappyNativeLibraryName = System.mapLibraryName("snappyjava");
/*     */     }
/* 430 */     if (snappyNativeLibraryPath != null) {
/* 431 */       File nativeLib = new File(snappyNativeLibraryPath, snappyNativeLibraryName);
/* 432 */       if (nativeLib.exists()) {
/* 433 */         return nativeLib;
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 438 */     snappyNativeLibraryPath = "/org/xerial/snappy/native/" + OSInfo.getNativeLibFolderPathForCurrentOS();
/*     */     
/* 440 */     if (SnappyLoader.class.getResource(snappyNativeLibraryPath + "/" + snappyNativeLibraryName) != null)
/*     */     {
/* 442 */       String tempFolder = new File(System.getProperty("org.xerial.snappy.tempdir", System.getProperty("java.io.tmpdir"))).getAbsolutePath();
/*     */       
/*     */ 
/*     */ 
/* 446 */       return extractLibraryFile(snappyNativeLibraryPath, snappyNativeLibraryName, tempFolder);
/*     */     }
/*     */     
/*     */ 
/* 450 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static String getVersion()
/*     */   {
/* 462 */     URL versionFile = SnappyLoader.class.getResource("/META-INF/maven/org.xerial.snappy/snappy-java/pom.properties");
/*     */     
/* 464 */     if (versionFile == null) {
/* 465 */       versionFile = SnappyLoader.class.getResource("/org/xerial/snappy/VERSION");
/*     */     }
/* 467 */     String version = "unknown";
/*     */     try {
/* 469 */       if (versionFile != null) {
/* 470 */         Properties versionData = new Properties();
/* 471 */         versionData.load(versionFile.openStream());
/* 472 */         version = versionData.getProperty("version", version);
/* 473 */         if (version.equals("unknown"))
/* 474 */           version = versionData.getProperty("VERSION", version);
/* 475 */         version = version.trim().replaceAll("[^0-9\\.]", "");
/*     */       }
/*     */     }
/*     */     catch (IOException e) {
/* 479 */       System.err.println(e);
/*     */     }
/* 481 */     return version;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\xerial\snappy\SnappyLoader.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */