package javax.wsdl.factory;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import javax.wsdl.Definition;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensionRegistry;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;

public abstract class WSDLFactory
{
  private static final String PROPERTY_NAME = "javax.wsdl.factory.WSDLFactory";
  private static final String PROPERTY_FILE_NAME = "wsdl.properties";
  private static final String DEFAULT_FACTORY_IMPL_NAME = "com.ibm.wsdl.factory.WSDLFactoryImpl";
  private static String fullPropertyFileName = null;
  
  public static WSDLFactory newInstance()
    throws WSDLException
  {
    String str = findFactoryImplName();
    return newInstance(str);
  }
  
  public static WSDLFactory newInstance(String paramString)
    throws WSDLException
  {
    if (paramString != null) {
      try
      {
        Class localClass = Class.forName(paramString);
        return (WSDLFactory)localClass.newInstance();
      }
      catch (Exception localException)
      {
        throw new WSDLException("CONFIGURATION_ERROR", "Problem instantiating factory implementation.", localException);
      }
    }
    throw new WSDLException("CONFIGURATION_ERROR", "Unable to find name of factory implementation.");
  }
  
  public abstract Definition newDefinition();
  
  public abstract WSDLReader newWSDLReader();
  
  public abstract WSDLWriter newWSDLWriter();
  
  public abstract ExtensionRegistry newPopulatedExtensionRegistry();
  
  private static String findFactoryImplName()
  {
    String str1 = null;
    try
    {
      str1 = System.getProperty("javax.wsdl.factory.WSDLFactory");
      if (str1 != null) {
        return str1;
      }
    }
    catch (SecurityException localSecurityException) {}
    String str2 = getFullPropertyFileName();
    if (str2 != null) {
      try
      {
        Properties localProperties = new Properties();
        File localFile = new File(str2);
        FileInputStream localFileInputStream = new FileInputStream(localFile);
        localProperties.load(localFileInputStream);
        localFileInputStream.close();
        str1 = localProperties.getProperty("javax.wsdl.factory.WSDLFactory");
        if (str1 != null) {
          return str1;
        }
      }
      catch (IOException localIOException) {}
    }
    return "com.ibm.wsdl.factory.WSDLFactoryImpl";
  }
  
  private static String getFullPropertyFileName()
  {
    if (fullPropertyFileName == null) {
      try
      {
        String str = System.getProperty("java.home");
        fullPropertyFileName = str + File.separator + "lib" + File.separator + "wsdl.properties";
      }
      catch (SecurityException localSecurityException) {}
    }
    return fullPropertyFileName;
  }
}


/* Location:              /Users/anandkum/Downloads/axis-wsdl4j-1.5.1.jar!/javax/wsdl/factory/WSDLFactory.class
 * Java compiler version: 2 (46.0)
 * JD-Core Version:       0.7.1
 */