package prefuse.data.search;

import java.io.IOException;
import java.util.logging.Logger;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.Hits;
import prefuse.data.Tuple;
import prefuse.util.StringLib;
import prefuse.util.collections.IntObjectHashMap;

public class KeywordSearchTupleSet
  extends SearchTupleSet
{
  private static final Logger s_logger = Logger.getLogger(KeywordSearchTupleSet.class.getName());
  protected IntObjectHashMap m_map = new IntObjectHashMap();
  protected String m_query = "";
  protected LuceneSearcher m_lucene = null;
  protected boolean m_storeTermVectors = false;
  protected int m_id = 1;
  
  public KeywordSearchTupleSet()
  {
    this.m_lucene = new LuceneSearcher();
  }
  
  public KeywordSearchTupleSet(LuceneSearcher paramLuceneSearcher)
  {
    this.m_lucene = paramLuceneSearcher;
  }
  
  public String getQuery()
  {
    return this.m_query;
  }
  
  public void search(String paramString)
  {
    if (paramString == null) {
      paramString = "";
    }
    if (paramString.equals(this.m_query)) {
      return;
    }
    Tuple[] arrayOfTuple1 = clearInternal();
    this.m_query = paramString;
    paramString.trim();
    if (paramString.length() == 0)
    {
      fireTupleEvent(null, -1);
      return;
    }
    this.m_lucene.setReadMode(true);
    try
    {
      Hits localHits = this.m_lucene.search(paramString);
      for (int i = 0; i < localHits.length(); i++)
      {
        Tuple localTuple = getMatchingTuple(localHits.doc(i));
        addInternal(localTuple);
      }
      Tuple[] arrayOfTuple2 = getTupleCount() > 0 ? toArray() : null;
      fireTupleEvent(arrayOfTuple2, arrayOfTuple1);
    }
    catch (ParseException localParseException)
    {
      s_logger.warning("Lucene query parse exception.\n" + StringLib.getStackTrace(localParseException));
    }
    catch (IOException localIOException)
    {
      s_logger.warning("Lucene IO exception.\n" + StringLib.getStackTrace(localIOException));
    }
  }
  
  protected Tuple getMatchingTuple(Document paramDocument)
  {
    int i = Integer.parseInt(paramDocument.get("prefuse-id"));
    return (Tuple)this.m_map.get(i);
  }
  
  public void index(Tuple paramTuple, String paramString)
  {
    this.m_lucene.setReadMode(false);
    String str;
    if ((str = paramTuple.getString(paramString)) == null) {
      return;
    }
    int i = this.m_id++;
    this.m_lucene.addDocument(getDocument(i, str));
    this.m_map.put(i, paramTuple);
  }
  
  public boolean isUnindexSupported()
  {
    return false;
  }
  
  public void unindex(Tuple paramTuple, String paramString)
  {
    throw new UnsupportedOperationException();
  }
  
  protected Document getDocument(int paramInt, String paramString)
  {
    Document localDocument = new Document();
    localDocument.add(Field.Text("prefuse-text", paramString, this.m_storeTermVectors));
    localDocument.add(Field.Keyword("prefuse-id", String.valueOf(paramInt)));
    return localDocument;
  }
  
  public LuceneSearcher getLuceneSearcher()
  {
    return this.m_lucene;
  }
  
  public IntObjectHashMap getTupleMap()
  {
    return (IntObjectHashMap)this.m_map.clone();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\search\KeywordSearchTupleSet.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */